@@ -93,12 +93,6 @@ struct ArrayFx {
   tiledb_encryption_type_t encryption_type_ = TILEDB_NO_ENCRYPTION;
   const char* encryption_key_ = nullptr;
 
-  // Serialization parameters
-  bool serialize_ = false;
-  bool refactored_query_v2_ = false;
-  // Buffers to allocate on server side for serialized queries
-  tiledb::test::ServerQueryBuffers server_buffers_;
-
   // Functions
   ArrayFx();
   ~ArrayFx();
@@ -391,9 +385,7 @@ void ArrayFx::write_fragment(tiledb_array_t* array, uint64_t timestamp) {
   CHECK(rc == TILEDB_OK);
   rc = tiledb_query_set_data_buffer(ctx_, query, "a", buffer, &buffer_size);
   CHECK(rc == TILEDB_OK);
-  rc = tiledb_query_submit(ctx_, query);
-  CHECK(rc == TILEDB_OK);
-  rc = tiledb_query_finalize(ctx_, query);
+  rc = tiledb_query_submit_and_finalize(ctx_, query);
   CHECK(rc == TILEDB_OK);
 
   // Clean up
@@ -470,7 +462,9 @@ TEST_CASE_METHOD(
 }
 
 TEST_CASE_METHOD(
-    ArrayFx, "C API: Test array with encryption", "[capi][array][encryption]") {
+    ArrayFx,
+    "C API: Test array with encryption",
+    "[capi][array][encryption][non-rest]") {
   // Create array schema
   tiledb_array_schema_t* array_schema;
   int rc = tiledb_array_schema_alloc(ctx_, TILEDB_SPARSE, &array_schema);
@@ -883,28 +877,22 @@ TEST_CASE_METHOD(
 TEST_CASE_METHOD(
     ArrayFx,
     "C API: Test opening array at timestamp, reads",
-    "[capi][array][open-at][reads]") {
+    "[capi][array][open-at][reads][rest]") {
   // TODO: refactor for each supported FS.
   std::string temp_dir = fs_vec_[0]->temp_dir();
+  std::string array_path = temp_dir + "array-open-at-reads";
+  std::string array_name = vfs_array_uri(fs_vec_[0], array_path);
 
-  std::string array_name = temp_dir + "array-open-at-reads";
   SECTION("- without encryption") {
     encryption_type_ = TILEDB_NO_ENCRYPTION;
     encryption_key_ = nullptr;
-    SECTION("no serialization") {
-      serialize_ = false;
-    }
-#ifdef TILEDB_SERIALIZATION
-    SECTION("serialization enabled") {
-      serialize_ = true;
-      refactored_query_v2_ = GENERATE(true, false);
-    }
-#endif
   }
 
-  SECTION("- with encryption") {
-    encryption_type_ = TILEDB_AES_256_GCM;
-    encryption_key_ = "0123456789abcdeF0123456789abcdeF";
+  if (!fs_vec_[0]->is_rest()) {
+    SECTION("- with encryption") {
+      encryption_type_ = TILEDB_AES_256_GCM;
+      encryption_key_ = "0123456789abcdeF0123456789abcdeF";
+    }
   }
 
   create_temp_dir(temp_dir);
@@ -951,13 +939,7 @@ TEST_CASE_METHOD(
   rc = tiledb_query_set_data_buffer(
       ctx_, query, "a", buffer_a1, &buffer_a1_size);
   CHECK(rc == TILEDB_OK);
-  rc = tiledb::test::submit_query_wrapper(
-      ctx_,
-      array_name,
-      &query,
-      server_buffers_,
-      serialize_,
-      refactored_query_v2_);
+  rc = tiledb_query_submit_and_finalize(ctx_, query);
   CHECK(rc == TILEDB_OK);
 
   // Close array and clean up
@@ -996,23 +978,24 @@ TEST_CASE_METHOD(
   rc = tiledb_array_open(ctx_, array, TILEDB_WRITE);
   CHECK(rc == TILEDB_OK);
 
+  // Create subarray
+  tiledb_subarray_t* sub;
+  rc = tiledb_subarray_alloc(ctx_, array, &sub);
+  CHECK(rc == TILEDB_OK);
+  rc = tiledb_subarray_set_subarray(ctx_, sub, subarray);
+  CHECK(rc == TILEDB_OK);
+
   // Submit query
   rc = tiledb_query_alloc(ctx_, array, TILEDB_WRITE, &query);
   CHECK(rc == TILEDB_OK);
   rc = tiledb_query_set_layout(ctx_, query, TILEDB_ROW_MAJOR);
   CHECK(rc == TILEDB_OK);
-  rc = tiledb_query_set_subarray(ctx_, query, subarray);
+  rc = tiledb_query_set_subarray_t(ctx_, query, sub);
   CHECK(rc == TILEDB_OK);
   rc = tiledb_query_set_data_buffer(
       ctx_, query, "a", buffer_upd, &buffer_upd_size);
   CHECK(rc == TILEDB_OK);
-  rc = tiledb::test::submit_query_wrapper(
-      ctx_,
-      array_name,
-      &query,
-      server_buffers_,
-      serialize_,
-      refactored_query_v2_);
+  rc = tiledb_query_submit(ctx_, query);
   CHECK(rc == TILEDB_OK);
 
   // Close array and clean up
@@ -1020,12 +1003,13 @@ TEST_CASE_METHOD(
   CHECK(rc == TILEDB_OK);
   tiledb_array_free(&array);
   tiledb_query_free(&query);
+  tiledb_subarray_free(&sub);
 
   std::vector<uint64_t> fragment_timestamps;
   rc = tiledb_vfs_ls(
       ctx_,
       vfs_,
-      get_commit_dir(array_name).c_str(),
+      get_commit_dir(array_path).c_str(),
       &get_fragment_timestamps,
       &fragment_timestamps);
   CHECK(rc == TILEDB_OK);
@@ -1059,24 +1043,24 @@ TEST_CASE_METHOD(
   rc = tiledb_array_open(ctx_, array, TILEDB_READ);
   CHECK(rc == TILEDB_OK);
 
-  // Submit query
+  // Create subarray
   int64_t subarray_read[] = {1, 10};
+  rc = tiledb_subarray_alloc(ctx_, array, &sub);
+  CHECK(rc == TILEDB_OK);
+  rc = tiledb_subarray_set_subarray(ctx_, sub, subarray_read);
+  CHECK(rc == TILEDB_OK);
+
+  // Submit query
   rc = tiledb_query_alloc(ctx_, array, TILEDB_READ, &query);
   CHECK(rc == TILEDB_OK);
   rc = tiledb_query_set_layout(ctx_, query, TILEDB_ROW_MAJOR);
   CHECK(rc == TILEDB_OK);
-  rc = tiledb_query_set_subarray(ctx_, query, subarray_read);
+  rc = tiledb_query_set_subarray_t(ctx_, query, sub);
   CHECK(rc == TILEDB_OK);
   rc = tiledb_query_set_data_buffer(
       ctx_, query, "a", buffer_read, &buffer_read_size);
   CHECK(rc == TILEDB_OK);
-  rc = tiledb::test::submit_query_wrapper(
-      ctx_,
-      array_name,
-      &query,
-      server_buffers_,
-      serialize_,
-      refactored_query_v2_);
+  rc = tiledb_query_submit(ctx_, query);
   CHECK(rc == TILEDB_OK);
 
   // Close array and clean up
@@ -1084,6 +1068,7 @@ TEST_CASE_METHOD(
   CHECK(rc == TILEDB_OK);
   tiledb_array_free(&array);
   tiledb_query_free(&query);
+  tiledb_subarray_free(&sub);
 
   // Check correctness
   int32_t buffer_read_c[] = {1, 2, 3, 4, 50, 60, 70, 8, 9, 10};
@@ -1124,23 +1109,23 @@ TEST_CASE_METHOD(
   CHECK(rc == TILEDB_OK);
   CHECK(timestamp_get == 0);
 
+  // Create subarray
+  rc = tiledb_subarray_alloc(ctx_, array, &sub);
+  CHECK(rc == TILEDB_OK);
+  rc = tiledb_subarray_set_subarray(ctx_, sub, subarray_read);
+  CHECK(rc == TILEDB_OK);
+
   // Submit query
   rc = tiledb_query_alloc(ctx_, array, TILEDB_READ, &query);
   CHECK(rc == TILEDB_OK);
   rc = tiledb_query_set_layout(ctx_, query, TILEDB_ROW_MAJOR);
   CHECK(rc == TILEDB_OK);
-  rc = tiledb_query_set_subarray(ctx_, query, subarray_read);
+  rc = tiledb_query_set_subarray_t(ctx_, query, sub);
   CHECK(rc == TILEDB_OK);
   rc = tiledb_query_set_data_buffer(
       ctx_, query, "a", buffer_read, &buffer_read_size);
   CHECK(rc == TILEDB_OK);
-  rc = tiledb::test::submit_query_wrapper(
-      ctx_,
-      array_name,
-      &query,
-      server_buffers_,
-      serialize_,
-      refactored_query_v2_);
+  rc = tiledb_query_submit(ctx_, query);
   CHECK(rc == TILEDB_OK);
 
   // Close array and clean up
@@ -1148,6 +1133,7 @@ TEST_CASE_METHOD(
   CHECK(rc == TILEDB_OK);
   tiledb_array_free(&array);
   tiledb_query_free(&query);
+  tiledb_subarray_free(&sub);
   tiledb_config_free(&cfg);
 
   // Check correctness
@@ -1181,23 +1167,23 @@ TEST_CASE_METHOD(
   rc = tiledb_array_open(ctx_, array, TILEDB_READ);
   CHECK(rc == TILEDB_OK);
 
+  // Create subarray
+  rc = tiledb_subarray_alloc(ctx_, array, &sub);
+  CHECK(rc == TILEDB_OK);
+  rc = tiledb_subarray_set_subarray(ctx_, sub, subarray_read);
+  CHECK(rc == TILEDB_OK);
+
   // Submit query
   rc = tiledb_query_alloc(ctx_, array, TILEDB_READ, &query);
   CHECK(rc == TILEDB_OK);
   rc = tiledb_query_set_layout(ctx_, query, TILEDB_ROW_MAJOR);
   CHECK(rc == TILEDB_OK);
-  rc = tiledb_query_set_subarray(ctx_, query, subarray_read);
+  rc = tiledb_query_set_subarray_t(ctx_, query, sub);
   CHECK(rc == TILEDB_OK);
   rc = tiledb_query_set_data_buffer(
       ctx_, query, "a", buffer_read, &buffer_read_size);
   CHECK(rc == TILEDB_OK);
-  rc = tiledb::test::submit_query_wrapper(
-      ctx_,
-      array_name,
-      &query,
-      server_buffers_,
-      serialize_,
-      refactored_query_v2_);
+  rc = tiledb_query_submit(ctx_, query);
   CHECK(rc == TILEDB_OK);
 
   // Close array and clean up
@@ -1205,6 +1191,7 @@ TEST_CASE_METHOD(
   CHECK(rc == TILEDB_OK);
   tiledb_array_free(&array);
   tiledb_query_free(&query);
+  tiledb_subarray_free(&sub);
   tiledb_config_free(&cfg);
 
   // Check correctness
@@ -1244,27 +1231,28 @@ TEST_CASE_METHOD(
   CHECK(rc == TILEDB_OK);
   CHECK(timestamp_get == fragment_timestamps[1]);
 
+  // Create subarray
+  rc = tiledb_subarray_alloc(ctx_, array, &sub);
+  CHECK(rc == TILEDB_OK);
+  rc = tiledb_subarray_set_subarray(ctx_, sub, subarray_read);
+  CHECK(rc == TILEDB_OK);
+
   // Submit query
   rc = tiledb_query_alloc(ctx_, array, TILEDB_READ, &query);
   CHECK(rc == TILEDB_OK);
   rc = tiledb_query_set_layout(ctx_, query, TILEDB_ROW_MAJOR);
   CHECK(rc == TILEDB_OK);
-  rc = tiledb_query_set_subarray(ctx_, query, subarray_read);
+  rc = tiledb_query_set_subarray_t(ctx_, query, sub);
   CHECK(rc == TILEDB_OK);
   rc = tiledb_query_set_data_buffer(
       ctx_, query, "a", buffer_read, &buffer_read_size);
   CHECK(rc == TILEDB_OK);
-  rc = tiledb::test::submit_query_wrapper(
-      ctx_,
-      array_name,
-      &query,
-      server_buffers_,
-      serialize_,
-      refactored_query_v2_);
+  rc = tiledb_query_submit(ctx_, query);
   CHECK(rc == TILEDB_OK);
 
   // Clean up but don't close the array yet (we will reopen it).
   tiledb_query_free(&query);
+  tiledb_subarray_free(&sub);
   tiledb_config_free(&cfg);
 
   // Check correctness
@@ -1281,27 +1269,28 @@ TEST_CASE_METHOD(
   rc = tiledb_array_reopen(ctx_, array);
   CHECK(rc == TILEDB_OK);
 
+  // Create subarray
+  rc = tiledb_subarray_alloc(ctx_, array, &sub);
+  CHECK(rc == TILEDB_OK);
+  rc = tiledb_subarray_set_subarray(ctx_, sub, subarray_read);
+  CHECK(rc == TILEDB_OK);
+
   // Submit query
   rc = tiledb_query_alloc(ctx_, array, TILEDB_READ, &query);
   CHECK(rc == TILEDB_OK);
   rc = tiledb_query_set_layout(ctx_, query, TILEDB_ROW_MAJOR);
   CHECK(rc == TILEDB_OK);
-  rc = tiledb_query_set_subarray(ctx_, query, subarray_read);
+  rc = tiledb_query_set_subarray_t(ctx_, query, sub);
   CHECK(rc == TILEDB_OK);
   rc = tiledb_query_set_data_buffer(
       ctx_, query, "a", buffer_read, &buffer_read_size);
   CHECK(rc == TILEDB_OK);
-  rc = tiledb::test::submit_query_wrapper(
-      ctx_,
-      array_name,
-      &query,
-      server_buffers_,
-      serialize_,
-      refactored_query_v2_);
+  rc = tiledb_query_submit(ctx_, query);
   CHECK(rc == TILEDB_OK);
 
   // Clean up but don't close the array yet (we will reopen it).
   tiledb_query_free(&query);
+  tiledb_subarray_free(&sub);
 
   // Check correctness
   int buffer_read_reopen_c[] = {1, 2, 3, 4, 5, 6, 7, 8, 9, 10};
@@ -1321,23 +1310,23 @@ TEST_CASE_METHOD(
   rc = tiledb_array_reopen(ctx_, array);
   CHECK(rc == TILEDB_OK);
 
+  // Create subarray
+  rc = tiledb_subarray_alloc(ctx_, array, &sub);
+  CHECK(rc == TILEDB_OK);
+  rc = tiledb_subarray_set_subarray(ctx_, sub, subarray_read);
+  CHECK(rc == TILEDB_OK);
+
   // Submit query
   rc = tiledb_query_alloc(ctx_, array, TILEDB_READ, &query);
   CHECK(rc == TILEDB_OK);
   rc = tiledb_query_set_layout(ctx_, query, TILEDB_ROW_MAJOR);
   CHECK(rc == TILEDB_OK);
-  rc = tiledb_query_set_subarray(ctx_, query, subarray_read);
+  rc = tiledb_query_set_subarray_t(ctx_, query, sub);
   CHECK(rc == TILEDB_OK);
   rc = tiledb_query_set_data_buffer(
       ctx_, query, "a", buffer_read, &buffer_read_size);
   CHECK(rc == TILEDB_OK);
-  rc = tiledb::test::submit_query_wrapper(
-      ctx_,
-      array_name,
-      &query,
-      server_buffers_,
-      serialize_,
-      refactored_query_v2_);
+  rc = tiledb_query_submit(ctx_, query);
   CHECK(rc == TILEDB_OK);
 
   // Close array and clean up
@@ -1345,6 +1334,7 @@ TEST_CASE_METHOD(
   CHECK(rc == TILEDB_OK);
   tiledb_query_free(&query);
   tiledb_array_free(&array);
+  tiledb_subarray_free(&sub);
 
   // Check correctness
   int buffer_read_reopen_start_c[] = {
@@ -1391,23 +1381,23 @@ TEST_CASE_METHOD(
   rc = tiledb_array_open(ctx_, array, TILEDB_READ);
   CHECK(rc == TILEDB_OK);
 
+  // Create subarray
+  rc = tiledb_subarray_alloc(ctx_, array, &sub);
+  CHECK(rc == TILEDB_OK);
+  rc = tiledb_subarray_set_subarray(ctx_, sub, subarray_read);
+  CHECK(rc == TILEDB_OK);
+
   // Submit query
   rc = tiledb_query_alloc(ctx_, array, TILEDB_READ, &query);
   CHECK(rc == TILEDB_OK);
   rc = tiledb_query_set_layout(ctx_, query, TILEDB_ROW_MAJOR);
   CHECK(rc == TILEDB_OK);
-  rc = tiledb_query_set_subarray(ctx_, query, subarray_read);
+  rc = tiledb_query_set_subarray_t(ctx_, query, sub);
   CHECK(rc == TILEDB_OK);
   rc = tiledb_query_set_data_buffer(
       ctx_, query, "a", buffer_read, &buffer_read_size);
   CHECK(rc == TILEDB_OK);
-  rc = tiledb::test::submit_query_wrapper(
-      ctx_,
-      array_name,
-      &query,
-      server_buffers_,
-      serialize_,
-      refactored_query_v2_);
+  rc = tiledb_query_submit(ctx_, query);
   CHECK(rc == TILEDB_OK);
 
   // Close array and clean up
@@ -1415,6 +1405,7 @@ TEST_CASE_METHOD(
   CHECK(rc == TILEDB_OK);
   tiledb_query_free(&query);
   tiledb_array_free(&array);
+  tiledb_subarray_free(&sub);
   tiledb_config_free(&cfg);
 
   // Check correctness
@@ -1461,23 +1452,23 @@ TEST_CASE_METHOD(
   rc = tiledb_array_open(ctx_, array, TILEDB_READ);
   CHECK(rc == TILEDB_OK);
 
+  // Create subarray
+  rc = tiledb_subarray_alloc(ctx_, array, &sub);
+  CHECK(rc == TILEDB_OK);
+  rc = tiledb_subarray_set_subarray(ctx_, sub, subarray_read);
+  CHECK(rc == TILEDB_OK);
+
   // Submit query
   rc = tiledb_query_alloc(ctx_, array, TILEDB_READ, &query);
   CHECK(rc == TILEDB_OK);
   rc = tiledb_query_set_layout(ctx_, query, TILEDB_ROW_MAJOR);
   CHECK(rc == TILEDB_OK);
-  rc = tiledb_query_set_subarray(ctx_, query, subarray_read);
+  rc = tiledb_query_set_subarray_t(ctx_, query, sub);
   CHECK(rc == TILEDB_OK);
   rc = tiledb_query_set_data_buffer(
       ctx_, query, "a", buffer_read, &buffer_read_size);
   CHECK(rc == TILEDB_OK);
-  rc = tiledb::test::submit_query_wrapper(
-      ctx_,
-      array_name,
-      &query,
-      server_buffers_,
-      serialize_,
-      refactored_query_v2_);
+  rc = tiledb_query_submit(ctx_, query);
   CHECK(rc == TILEDB_OK);
 
   // Close array and clean up
@@ -1485,6 +1476,7 @@ TEST_CASE_METHOD(
   CHECK(rc == TILEDB_OK);
   tiledb_query_free(&query);
   tiledb_array_free(&array);
+  tiledb_subarray_free(&sub);
   tiledb_config_free(&cfg);
 
   // Check correctness
@@ -1511,28 +1503,22 @@ TEST_CASE_METHOD(
 TEST_CASE_METHOD(
     ArrayFx,
     "C API: Test opening array at timestamp, writes",
-    "[capi][array][open-at][writes]") {
+    "[capi][array][open-at][writes][rest-fails][sc-42722]") {
   // TODO: refactor for each supported FS.
   std::string temp_dir = fs_vec_[0]->temp_dir();
+  std::string array_name =
+      vfs_array_uri(fs_vec_[0], temp_dir + "array-open-at-writes");
 
-  std::string array_name = temp_dir + "array-open-at-writes";
   SECTION("- without encryption") {
     encryption_type_ = TILEDB_NO_ENCRYPTION;
     encryption_key_ = nullptr;
-    SECTION("no serialization") {
-      serialize_ = false;
-    }
-#ifdef TILEDB_SERIALIZATION
-    SECTION("serialization enabled") {
-      serialize_ = true;
-      refactored_query_v2_ = GENERATE(true, false);
-    }
-#endif
   }
 
-  SECTION("- with encryption") {
-    encryption_type_ = TILEDB_AES_256_GCM;
-    encryption_key_ = "0123456789abcdeF0123456789abcdeF";
+  if (!fs_vec_[0]->is_rest()) {
+    SECTION("- with encryption") {
+      encryption_type_ = TILEDB_AES_256_GCM;
+      encryption_key_ = "0123456789abcdeF0123456789abcdeF";
+    }
   }
 
   create_temp_dir(temp_dir);
@@ -1584,13 +1570,7 @@ TEST_CASE_METHOD(
   rc = tiledb_query_set_data_buffer(
       ctx_, query, "a", buffer_a1, &buffer_a1_size);
   CHECK(rc == TILEDB_OK);
-  rc = tiledb::test::submit_query_wrapper(
-      ctx_,
-      array_name,
-      &query,
-      server_buffers_,
-      serialize_,
-      refactored_query_v2_);
+  rc = tiledb_query_submit_and_finalize(ctx_, query);
   CHECK(rc == TILEDB_OK);
 
   // Get written timestamp
@@ -1642,26 +1622,27 @@ TEST_CASE_METHOD(
   CHECK(rc == TILEDB_OK);
   CHECK(timestamp_get == 0);
 
+  // Create subarray
+  int64_t subarray_read[] = {1, 10};
+  tiledb_subarray_t* sub;
+  rc = tiledb_subarray_alloc(ctx_, array, &sub);
+  CHECK(rc == TILEDB_OK);
+  rc = tiledb_subarray_set_subarray(ctx_, sub, subarray_read);
+  CHECK(rc == TILEDB_OK);
+
   // Submit query
   rc = tiledb_query_alloc(ctx_, array, TILEDB_READ, &query);
   CHECK(rc == TILEDB_OK);
   rc = tiledb_query_set_layout(ctx_, query, TILEDB_ROW_MAJOR);
   CHECK(rc == TILEDB_OK);
-  int64_t subarray_read[] = {1, 10};
-  rc = tiledb_query_set_subarray(ctx_, query, subarray_read);
+  rc = tiledb_query_set_subarray_t(ctx_, query, sub);
   CHECK(rc == TILEDB_OK);
   int buffer_read[10];
   uint64_t buffer_read_size = sizeof(buffer_read);
   rc = tiledb_query_set_data_buffer(
       ctx_, query, "a", buffer_read, &buffer_read_size);
   CHECK(rc == TILEDB_OK);
-  rc = tiledb::test::submit_query_wrapper(
-      ctx_,
-      array_name,
-      &query,
-      server_buffers_,
-      serialize_,
-      refactored_query_v2_);
+  rc = tiledb_query_submit(ctx_, query);
   CHECK(rc == TILEDB_OK);
 
   // Close array and clean up
@@ -1669,6 +1650,7 @@ TEST_CASE_METHOD(
   CHECK(rc == TILEDB_OK);
   tiledb_array_free(&array);
   tiledb_query_free(&query);
+  tiledb_subarray_free(&sub);
   tiledb_config_free(&cfg);
 
   // Check correctness
@@ -1702,23 +1684,23 @@ TEST_CASE_METHOD(
   rc = tiledb_array_open(ctx_, array, TILEDB_READ);
   CHECK(rc == TILEDB_OK);
 
+  // Create subarray
+  rc = tiledb_subarray_alloc(ctx_, array, &sub);
+  CHECK(rc == TILEDB_OK);
+  rc = tiledb_subarray_set_subarray(ctx_, sub, subarray_read);
+  CHECK(rc == TILEDB_OK);
+
   // Submit query
   rc = tiledb_query_alloc(ctx_, array, TILEDB_READ, &query);
   CHECK(rc == TILEDB_OK);
   rc = tiledb_query_set_layout(ctx_, query, TILEDB_ROW_MAJOR);
   CHECK(rc == TILEDB_OK);
-  rc = tiledb_query_set_subarray(ctx_, query, subarray_read);
+  rc = tiledb_query_set_subarray_t(ctx_, query, sub);
   CHECK(rc == TILEDB_OK);
   rc = tiledb_query_set_data_buffer(
       ctx_, query, "a", buffer_read, &buffer_read_size);
   CHECK(rc == TILEDB_OK);
-  rc = tiledb::test::submit_query_wrapper(
-      ctx_,
-      array_name,
-      &query,
-      server_buffers_,
-      serialize_,
-      refactored_query_v2_);
+  rc = tiledb_query_submit(ctx_, query);
   CHECK(rc == TILEDB_OK);
 
   // Close array and clean up
@@ -1726,6 +1708,7 @@ TEST_CASE_METHOD(
   CHECK(rc == TILEDB_OK);
   tiledb_array_free(&array);
   tiledb_query_free(&query);
+  tiledb_subarray_free(&sub);
   tiledb_config_free(&cfg);
 
   // Check correctness
@@ -1739,10 +1722,10 @@ TEST_CASE_METHOD(
 TEST_CASE_METHOD(
     ArrayFx,
     "C API: Check writing coordinates out of bounds",
-    "[capi][array][array-write-coords-oob]") {
-  SupportedFsLocal local_fs;
-  std::string temp_dir = local_fs.file_prefix() + local_fs.temp_dir();
-  std::string array_name = temp_dir + "array-write-coords-oob";
+    "[capi][array][array-write-coords-oob][rest]") {
+  std::string temp_dir = fs_vec_[0]->temp_dir();
+  std::string array_name =
+      vfs_array_uri(fs_vec_[0], temp_dir + "array-write-coords-oob");
   create_temp_dir(temp_dir);
 
   int dimension = 0;
@@ -1756,7 +1739,6 @@ TEST_CASE_METHOD(
   // Create TileDB context
   tiledb_config_t* cfg = nullptr;
   tiledb_error_t* err = nullptr;
-  tiledb_ctx_t* ctx = nullptr;
 
   SECTION("- Check out-of-bounds coordinates") {
     check_coords_oob = true;
@@ -1842,59 +1824,58 @@ TEST_CASE_METHOD(
     }
   }
 
-  REQUIRE(tiledb_ctx_alloc(cfg, &ctx) == TILEDB_OK);
-  REQUIRE(err == nullptr);
+  tiledb_ctx_free(&ctx_);
+  tiledb_vfs_free(&vfs_);
+  // reallocate with input config
+  vfs_test_init(fs_vec_, &ctx_, &vfs_, cfg).ok();
   tiledb_config_free(&cfg);
 
   // Open array
   tiledb_array_t* array;
-  rc = tiledb_array_alloc(ctx, array_name.c_str(), &array);
+  rc = tiledb_array_alloc(ctx_, array_name.c_str(), &array);
   CHECK(rc == TILEDB_OK);
-  rc = tiledb_array_open(ctx, array, TILEDB_WRITE);
+  rc = tiledb_array_open(ctx_, array, TILEDB_WRITE);
   CHECK(rc == TILEDB_OK);
 
   // Submit query
   tiledb_query_t* query;
-  rc = tiledb_query_alloc(ctx, array, TILEDB_WRITE, &query);
+  rc = tiledb_query_alloc(ctx_, array, TILEDB_WRITE, &query);
   CHECK(rc == TILEDB_OK);
-  rc = tiledb_query_set_layout(ctx, query, TILEDB_GLOBAL_ORDER);
-  CHECK(rc == TILEDB_OK);
-  rc =
-      tiledb_query_set_data_buffer(ctx, query, "a", buffer_a1, &buffer_a1_size);
+  rc = tiledb_query_set_layout(ctx_, query, TILEDB_GLOBAL_ORDER);
   CHECK(rc == TILEDB_OK);
   rc = tiledb_query_set_data_buffer(
-      ctx, query, "d1", buffer_coords_dim1, &buffer_coords_size);
+      ctx_, query, "a", buffer_a1, &buffer_a1_size);
+  CHECK(rc == TILEDB_OK);
+  rc = tiledb_query_set_data_buffer(
+      ctx_, query, "d1", buffer_coords_dim1, &buffer_coords_size);
   CHECK(rc == TILEDB_OK);
   if (dimension == 2) {
     rc = tiledb_query_set_data_buffer(
-        ctx, query, "d2", buffer_coords_dim2, &buffer_coords_size);
+        ctx_, query, "d2", buffer_coords_dim2, &buffer_coords_size);
     CHECK(rc == TILEDB_OK);
   }
-  rc = tiledb_query_submit(ctx, query);
+  rc = tiledb_query_submit_and_finalize(ctx_, query);
   if (check_coords_oob) {
     CHECK(rc == TILEDB_ERR);
   } else {
     CHECK(rc == TILEDB_OK);
   }
-  rc = tiledb_query_finalize(ctx, query);
-  CHECK(rc == TILEDB_OK);
 
   // Close array and clean up
-  rc = tiledb_array_close(ctx, array);
+  rc = tiledb_array_close(ctx_, array);
   CHECK(rc == TILEDB_OK);
   tiledb_array_free(&array);
   tiledb_query_free(&query);
-  tiledb_ctx_free(&ctx);
 
   remove_temp_dir(temp_dir);
 }
 
 TEST_CASE_METHOD(
-    ArrayFx, "C API: Test empty array", "[capi][array][array-empty]") {
-  SupportedFsLocal local_fs;
-  std::string array_name =
-      local_fs.file_prefix() + local_fs.temp_dir() + "array_empty";
-  create_temp_dir(local_fs.file_prefix() + local_fs.temp_dir());
+    ArrayFx, "C API: Test empty array", "[capi][array][array-empty][rest]") {
+  std::string temp_dir = fs_vec_[0]->temp_dir();
+  std::string array_name = vfs_array_uri(fs_vec_[0], temp_dir + "array_empty");
+
+  create_temp_dir(temp_dir);
 
   create_sparse_vector(array_name);
 
@@ -1937,15 +1918,17 @@ TEST_CASE_METHOD(
   tiledb_array_free(&array);
   tiledb_query_free(&query);
 
-  remove_temp_dir(local_fs.file_prefix() + local_fs.temp_dir());
+  remove_temp_dir(temp_dir);
 }
 
 TEST_CASE_METHOD(
-    ArrayFx, "C API: Test deletion of array", "[capi][array][delete]") {
-  SupportedFsLocal local_fs;
-  std::string array_name =
-      local_fs.file_prefix() + local_fs.temp_dir() + "array_delete";
-  create_temp_dir(local_fs.file_prefix() + local_fs.temp_dir());
+    ArrayFx, "C API: Test deletion of array", "[capi][array][delete][rest]") {
+  std::string temp_dir = fs_vec_[0]->temp_dir();
+  std::string array_path = temp_dir + "array_delete";
+  std::string array_name = vfs_array_uri(fs_vec_[0], array_path);
+
+  create_temp_dir(temp_dir);
+
   create_dense_vector(array_name);
 
   // Conditionally consolidate
@@ -1961,11 +1944,17 @@ TEST_CASE_METHOD(
   write_fragment(array, 5);
 
   // Check write
-  CHECK(tiledb::test::num_commits(array_name) == 3);
-  CHECK(tiledb::test::num_fragments(array_name) == 3);
+  // Get the number of write files in the commit directory
+  tiledb::Context ctx(ctx_, false);
+  tiledb::VFS vfs(ctx);
+  CommitsDirectory commits_dir(vfs, array_path);
+  CHECK(commits_dir.file_count(constants::write_file_suffix) == 3);
+  auto uris = vfs.ls(
+      array_path + "/" + tiledb::sm::constants::array_fragments_dir_name);
+  CHECK(static_cast<uint32_t>(uris.size()) == 3);
 
   // Conditionally consolidate commits
-  if (consolidate) {
+  if (consolidate && !fs_vec_[0]->is_rest()) {
     tiledb_config_t* cfg;
     tiledb_error_t* err = nullptr;
     rc = tiledb_config_alloc(&cfg, &err);
@@ -1979,16 +1968,22 @@ TEST_CASE_METHOD(
     tiledb_config_free(&cfg);
 
     // Validate working directory
-    CHECK(tiledb::test::num_commits(array_name) == 3);
-    CHECK(tiledb::test::num_fragments(array_name) == 3);
+    CommitsDirectory commits_dir2(vfs, array_path);
+    CHECK(commits_dir2.file_count(constants::write_file_suffix) == 3);
+    auto uris2 = vfs.ls(
+        array_path + "/" + tiledb::sm::constants::array_fragments_dir_name);
+    CHECK(static_cast<uint32_t>(uris2.size()) == 3);
   }
 
   // Delete array data
   rc = tiledb_array_delete(ctx_, array_name.c_str());
 
   // Validate working directory after delete
-  CHECK(tiledb::test::num_commits(array_name) == 0);
-  CHECK(tiledb::test::num_fragments(array_name) == 0);
+  CommitsDirectory commits_dir3(vfs, array_path);
+  CHECK(commits_dir3.file_count(constants::write_file_suffix) == 0);
+  auto uris3 = vfs.ls(
+      array_path + "/" + tiledb::sm::constants::array_fragments_dir_name);
+  CHECK(static_cast<uint32_t>(uris3.size()) == 0);
 
   // Try to open array
   rc = tiledb_array_open(ctx_, array, TILEDB_READ);
@@ -1996,18 +1991,18 @@ TEST_CASE_METHOD(
 
   // Clean up
   tiledb_array_free(&array);
-  remove_temp_dir(local_fs.file_prefix() + local_fs.temp_dir());
+  remove_temp_dir(temp_dir);
 }
 
 TEST_CASE_METHOD(
     ArrayFx,
-    "C API: Test query errors, getting subarray info from write queries in "
+    "C API: Test subarray errors, getting subarray info from write queries in "
     "sparse arrays",
-    "[capi][query][error][sparse]") {
-  SupportedFsLocal local_fs;
+    "[capi][subarray][error][sparse][rest]") {
+  std::string temp_dir = fs_vec_[0]->temp_dir();
   std::string array_name =
-      local_fs.file_prefix() + local_fs.temp_dir() + "query_error_sparse";
-  create_temp_dir(local_fs.file_prefix() + local_fs.temp_dir());
+      vfs_array_uri(fs_vec_[0], temp_dir + "query_error_sparse");
+  create_temp_dir(temp_dir);
 
   create_sparse_vector(array_name);
 
@@ -2021,17 +2016,20 @@ TEST_CASE_METHOD(
   // Prepare query
   tiledb_query_t* query;
   rc = tiledb_query_alloc(ctx_, array, TILEDB_WRITE, &query);
+
+  // Prepare subarray
+  tiledb_subarray_t* subarray;
+  rc = tiledb_subarray_alloc(ctx_, array, &subarray);
   CHECK(rc == TILEDB_OK);
   uint64_t range_num;
 
-  // Note: deprecated function.
-  rc = tiledb_query_get_range_num(ctx_, query, 0, &range_num);
+  rc = tiledb_subarray_get_range_num(ctx_, subarray, 0, &range_num);
   CHECK(rc == TILEDB_OK);
   CHECK(range_num == 1);
 
   // Note: deprecated function.
   const void *start, *end, *stride;
-  rc = tiledb_query_get_range(ctx_, query, 0, 0, &start, &end, &stride);
+  rc = tiledb_subarray_get_range(ctx_, subarray, 0, 0, &start, &end, &stride);
   CHECK(rc == TILEDB_OK);
   CHECK(*static_cast<const int64_t*>(start) == -1);
   CHECK(*static_cast<const int64_t*>(end) == 2);
@@ -2039,10 +2037,12 @@ TEST_CASE_METHOD(
 
   int64_t s = 10;
   int64_t e = 20;
-  rc = tiledb_query_add_range(ctx_, query, 0, &s, &e, nullptr);
+  rc = tiledb_subarray_add_range(ctx_, subarray, 0, &s, &e, nullptr);
   CHECK(rc == TILEDB_ERR);
-  int64_t subarray[] = {-1, 2};
-  rc = tiledb_query_set_subarray(ctx_, query, subarray);
+  int64_t sub[] = {-1, 2};
+  rc = tiledb_subarray_set_subarray(ctx_, subarray, sub);
+  CHECK(rc == TILEDB_OK);
+  rc = tiledb_query_set_subarray_t(ctx_, query, subarray);
   CHECK(rc == TILEDB_ERR);
 
   // Close array
@@ -2052,18 +2052,19 @@ TEST_CASE_METHOD(
   // Clean up
   tiledb_array_free(&array);
   tiledb_query_free(&query);
+  tiledb_subarray_free(&subarray);
 
-  remove_temp_dir(local_fs.file_prefix() + local_fs.temp_dir());
+  remove_temp_dir(temp_dir);
 }
 
 TEST_CASE_METHOD(
     ArrayFx,
     "C API: Test query errors, dense writes",
-    "[capi][query][error][dense]") {
-  SupportedFsLocal local_fs;
+    "[capi][query][error][dense][rest]") {
+  std::string temp_dir = fs_vec_[0]->temp_dir();
   std::string array_name =
-      local_fs.file_prefix() + local_fs.temp_dir() + "query_error_dense";
-  create_temp_dir(local_fs.file_prefix() + local_fs.temp_dir());
+      vfs_array_uri(fs_vec_[0], temp_dir + "query_error_dense");
+  create_temp_dir(temp_dir);
 
   create_dense_array(array_name);
 
@@ -2077,45 +2078,50 @@ TEST_CASE_METHOD(
   int32_t a[] = {1, 2, 3, 4};
   uint64_t a_size = sizeof(a);
 
-  // Prepare query
+  // Prepare query/subarray
   tiledb_query_t* query;
   rc = tiledb_query_alloc(ctx_, array, TILEDB_WRITE, &query);
   CHECK(rc == TILEDB_OK);
+  tiledb_subarray_t* subarray;
+  rc = tiledb_subarray_alloc(ctx_, array, &subarray);
+  CHECK(rc == TILEDB_OK);
   rc = tiledb_query_set_data_buffer(ctx_, query, "a", a, &a_size);
   CHECK(rc == TILEDB_OK);
   uint64_t range_num;
-  rc = tiledb_query_get_range_num(ctx_, query, 0, &range_num);
+  rc = tiledb_subarray_get_range_num(ctx_, subarray, 0, &range_num);
   CHECK(rc == TILEDB_OK);
   CHECK(range_num == 1);  // The default
   const void *start, *end, *stride;
-  rc = tiledb_query_get_range(ctx_, query, 0, 0, &start, &end, &stride);
+  rc = tiledb_subarray_get_range(ctx_, subarray, 0, 0, &start, &end, &stride);
   CHECK(rc == TILEDB_OK);
   CHECK(*(const uint64_t*)start == 1);
   CHECK(*(const uint64_t*)end == 10);
   int64_t s = 1;
   int64_t e = 2;
-  rc = tiledb_query_add_range(ctx_, query, 0, &s, &e, nullptr);
+  rc = tiledb_subarray_add_range(ctx_, subarray, 0, &s, &e, nullptr);
   CHECK(rc == TILEDB_OK);
 
-  int64_t subarray[] = {2, 3, 4, 5};
-  rc = tiledb_query_set_subarray(ctx_, query, subarray);
+  int64_t sub[] = {2, 3, 4, 5};
+  rc = tiledb_subarray_set_subarray(ctx_, subarray, sub);
   CHECK(rc == TILEDB_OK);
-  rc = tiledb_query_add_range(ctx_, query, 0, &s, &e, nullptr);
+  rc = tiledb_subarray_add_range(ctx_, subarray, 0, &s, &e, nullptr);
   CHECK(rc == TILEDB_OK);
-  rc = tiledb_query_get_range_num(ctx_, query, 0, &range_num);
+  rc = tiledb_subarray_get_range_num(ctx_, subarray, 0, &range_num);
   CHECK(rc == TILEDB_OK);
   CHECK(range_num == 2);
-  rc = tiledb_query_get_range(ctx_, query, 0, 0, &start, &end, &stride);
+  rc = tiledb_subarray_get_range(ctx_, subarray, 0, 0, &start, &end, &stride);
   CHECK(rc == TILEDB_OK);
   CHECK(*(const uint64_t*)start == 2);
   CHECK(*(const uint64_t*)end == 3);
-  rc = tiledb_query_get_range(ctx_, query, 1, 0, &start, &end, &stride);
+  rc = tiledb_subarray_get_range(ctx_, subarray, 1, 0, &start, &end, &stride);
   CHECK(rc == TILEDB_OK);
   CHECK(*(const uint64_t*)start == 4);
   CHECK(*(const uint64_t*)end == 5);
 
   rc = tiledb_query_set_layout(ctx_, query, TILEDB_GLOBAL_ORDER);
   CHECK(rc == TILEDB_OK);
+  rc = tiledb_query_set_subarray_t(ctx_, query, subarray);
+  CHECK(rc == TILEDB_OK);
   rc = tiledb_query_submit(ctx_, query);
   CHECK(rc == TILEDB_ERR);
 
@@ -2126,18 +2132,20 @@ TEST_CASE_METHOD(
   // Clean up
   tiledb_array_free(&array);
   tiledb_query_free(&query);
+  tiledb_subarray_free(&subarray);
 
-  remove_temp_dir(local_fs.file_prefix() + local_fs.temp_dir());
+  remove_temp_dir(temp_dir);
 }
 
 TEST_CASE_METHOD(
     ArrayFx,
     "C API: Test query errors, dense unordered writes",
-    "[capi][query][error][dense]") {
-  SupportedFsLocal local_fs;
+    "[capi][query][error][dense][rest]") {
+  std::string temp_dir = fs_vec_[0]->temp_dir();
   std::string array_name =
-      local_fs.file_prefix() + local_fs.temp_dir() + "query_error_dense";
-  create_temp_dir(local_fs.file_prefix() + local_fs.temp_dir());
+      vfs_array_uri(fs_vec_[0], temp_dir + "query_error_dense");
+
+  create_temp_dir(temp_dir);
 
   create_dense_array(array_name);
 
@@ -2164,17 +2172,18 @@ TEST_CASE_METHOD(
   tiledb_array_free(&array);
   tiledb_query_free(&query);
 
-  remove_temp_dir(local_fs.file_prefix() + local_fs.temp_dir());
+  remove_temp_dir(temp_dir);
 }
 
 TEST_CASE_METHOD(
     ArrayFx,
     "C API: Test query errors, dense reads in global order",
-    "[capi][query][error][dense]") {
-  SupportedFsLocal local_fs;
+    "[capi][query][error][dense][rest]") {
+  std::string temp_dir = fs_vec_[0]->temp_dir();
   std::string array_name =
-      local_fs.file_prefix() + local_fs.temp_dir() + "query_error_dense";
-  create_temp_dir(local_fs.file_prefix() + local_fs.temp_dir());
+      vfs_array_uri(fs_vec_[0], temp_dir + "query_error_dense");
+
+  create_temp_dir(temp_dir);
 
   create_dense_array(array_name);
 
@@ -2195,16 +2204,21 @@ TEST_CASE_METHOD(
   rc = tiledb_query_set_data_buffer(ctx_, query, "a", a, &a_size);
   CHECK(rc == TILEDB_OK);
 
-  int64_t subarray[] = {2, 3, 4, 5};
-  rc = tiledb_query_set_subarray(ctx_, query, subarray);
+  tiledb_subarray_t* subarray;
+  rc = tiledb_subarray_alloc(ctx_, array, &subarray);
+  CHECK(rc == TILEDB_OK);
+  int64_t sub[] = {2, 3, 4, 5};
+  rc = tiledb_subarray_set_subarray(ctx_, subarray, sub);
   CHECK(rc == TILEDB_OK);
   int64_t s = 1;
   int64_t e = 2;
-  rc = tiledb_query_add_range(ctx_, query, 0, &s, &e, nullptr);
+  rc = tiledb_subarray_add_range(ctx_, subarray, 0, &s, &e, nullptr);
   CHECK(rc == TILEDB_OK);
 
   rc = tiledb_query_set_layout(ctx_, query, TILEDB_GLOBAL_ORDER);
   CHECK(rc == TILEDB_OK);
+  rc = tiledb_query_set_subarray_t(ctx_, query, subarray);
+  CHECK(rc == TILEDB_OK);
   rc = tiledb_query_submit(ctx_, query);
   CHECK(rc == TILEDB_ERR);
 
@@ -2215,8 +2229,9 @@ TEST_CASE_METHOD(
   // Clean up
   tiledb_array_free(&array);
   tiledb_query_free(&query);
+  tiledb_subarray_free(&subarray);
 
-  remove_temp_dir(local_fs.file_prefix() + local_fs.temp_dir());
+  remove_temp_dir(temp_dir);
 }
 
 TEST_CASE_METHOD(
@@ -2590,13 +2605,12 @@ TEST_CASE_METHOD(
       1,
       &buff);
   REQUIRE(rc == TILEDB_OK);
-  auto st = tiledb::sm::serialization::array_deserialize(
+  tiledb::sm::serialization::array_deserialize(
       array->array_.get(),
       tiledb::sm::SerializationType::CAPNP,
       buff->buffer(),
-      ctx_->storage_manager(),
+      ctx_->context().resources(),
       memory_tracker_);
-  REQUIRE(st.ok());
 
   // 6. Server: Close array and clean up
   rc = tiledb_array_close(ctx_, deserialized_array_server);
