@@ -61,9 +61,9 @@ struct SparseHeterFx {
 
   // Serialization parameters
   bool serialize_ = false;
-  bool refactored_query_v2_ = false;
-  // Buffers to allocate on server side for serialized queries
-  ServerQueryBuffers server_buffers_;
+
+  // Path to prepend to array name according to filesystem/mode
+  std::string prefix_;
 
   // Functions
   SparseHeterFx();
@@ -147,9 +147,13 @@ SparseHeterFx::SparseHeterFx()
     : fs_vec_(vfs_test_get_fs_vec()) {
   // Initialize vfs test
   REQUIRE(vfs_test_init(fs_vec_, &ctx_, &vfs_).ok());
+  auto temp_dir = fs_vec_[0]->temp_dir();
+  create_temp_dir(temp_dir);
+  prefix_ = vfs_array_uri(fs_vec_[0], temp_dir);
 }
 
 SparseHeterFx::~SparseHeterFx() {
+  remove_temp_dir(fs_vec_[0]->temp_dir());
   // Close vfs test
   REQUIRE(vfs_test_close(fs_vec_, ctx_, vfs_).ok());
   tiledb_vfs_free(&vfs_);
@@ -420,11 +424,16 @@ void SparseHeterFx::check_est_result_size_float_int64(
   tiledb_query_t* query;
   rc = tiledb_query_alloc(ctx_, array, TILEDB_READ, &query);
   CHECK(rc == TILEDB_OK);
-  rc = tiledb_query_add_range(
-      ctx_, query, 0, &subarray_f[0], &subarray_f[1], nullptr);
+  tiledb_subarray_t* subarray;
+  rc = tiledb_subarray_alloc(ctx_, array, &subarray);
   CHECK(rc == TILEDB_OK);
-  rc = tiledb_query_add_range(
-      ctx_, query, 1, &subarray_i[0], &subarray_i[1], nullptr);
+  rc = tiledb_subarray_add_range(
+      ctx_, subarray, 0, &subarray_f[0], &subarray_f[1], nullptr);
+  CHECK(rc == TILEDB_OK);
+  rc = tiledb_subarray_add_range(
+      ctx_, subarray, 1, &subarray_i[0], &subarray_i[1], nullptr);
+  CHECK(rc == TILEDB_OK);
+  rc = tiledb_query_set_subarray_t(ctx_, query, subarray);
   CHECK(rc == TILEDB_OK);
 
   // Check error for zipped coordinates
@@ -448,6 +457,7 @@ void SparseHeterFx::check_est_result_size_float_int64(
   // Clean up
   tiledb_array_free(&array);
   tiledb_query_free(&query);
+  tiledb_subarray_free(&subarray);
 }
 
 void SparseHeterFx::check_est_result_size_int64_float(
@@ -466,11 +476,16 @@ void SparseHeterFx::check_est_result_size_int64_float(
   tiledb_query_t* query;
   rc = tiledb_query_alloc(ctx_, array, TILEDB_READ, &query);
   CHECK(rc == TILEDB_OK);
-  rc = tiledb_query_add_range(
-      ctx_, query, 0, &subarray_i[0], &subarray_i[1], nullptr);
+  tiledb_subarray_t* subarray;
+  rc = tiledb_subarray_alloc(ctx_, array, &subarray);
   CHECK(rc == TILEDB_OK);
-  rc = tiledb_query_add_range(
-      ctx_, query, 1, &subarray_f[0], &subarray_f[1], nullptr);
+  rc = tiledb_subarray_add_range(
+      ctx_, subarray, 0, &subarray_i[0], &subarray_i[1], nullptr);
+  CHECK(rc == TILEDB_OK);
+  rc = tiledb_subarray_add_range(
+      ctx_, subarray, 1, &subarray_f[0], &subarray_f[1], nullptr);
+  CHECK(rc == TILEDB_OK);
+  rc = tiledb_query_set_subarray_t(ctx_, query, subarray);
   CHECK(rc == TILEDB_OK);
 
   // Check error for zipped coordinates
@@ -494,6 +509,7 @@ void SparseHeterFx::check_est_result_size_int64_float(
   // Clean up
   tiledb_array_free(&array);
   tiledb_query_free(&query);
+  tiledb_subarray_free(&subarray);
 }
 
 void SparseHeterFx::write_sparse_array_float_int64(
@@ -529,13 +545,11 @@ void SparseHeterFx::write_sparse_array_float_int64(
   REQUIRE(rc == TILEDB_OK);
 
   // Submit query
-  rc = submit_query_wrapper(
-      ctx_,
-      array_name,
-      &query,
-      server_buffers_,
-      serialize_,
-      refactored_query_v2_);
+  if (layout == TILEDB_GLOBAL_ORDER) {
+    rc = tiledb_query_submit_and_finalize(ctx_, query);
+  } else {
+    rc = tiledb_query_submit(ctx_, query);
+  }
   REQUIRE(rc == TILEDB_OK);
 
   // Close array
@@ -578,9 +592,12 @@ void SparseHeterFx::write_sparse_array_int64_float(
   REQUIRE(rc == TILEDB_OK);
   rc = tiledb_query_set_layout(ctx_, query, layout);
   REQUIRE(rc == TILEDB_OK);
-  rc = tiledb_query_submit(ctx_, query);
-  REQUIRE(rc == TILEDB_OK);
-  rc = tiledb_query_finalize(ctx_, query);
+
+  if (layout == TILEDB_GLOBAL_ORDER) {
+    rc = tiledb_query_submit_and_finalize(ctx_, query);
+  } else {
+    rc = tiledb_query_submit(ctx_, query);
+  }
   REQUIRE(rc == TILEDB_OK);
 
   // Close array
@@ -626,12 +643,17 @@ void SparseHeterFx::check_read_sparse_array_float_int64(
   REQUIRE(rc == TILEDB_OK);
   rc = tiledb_query_set_layout(ctx_, query, layout);
   REQUIRE(rc == TILEDB_OK);
-  rc = tiledb_query_add_range(
-      ctx_, query, 0, &subarray_f[0], &subarray_f[1], nullptr);
+  tiledb_subarray_t* subarray;
+  rc = tiledb_subarray_alloc(ctx_, array, &subarray);
+  CHECK(rc == TILEDB_OK);
+  rc = tiledb_subarray_add_range(
+      ctx_, subarray, 0, &subarray_f[0], &subarray_f[1], nullptr);
   REQUIRE(rc == TILEDB_OK);
-  rc = tiledb_query_add_range(
-      ctx_, query, 1, &subarray_i[0], &subarray_i[1], nullptr);
+  rc = tiledb_subarray_add_range(
+      ctx_, subarray, 1, &subarray_i[0], &subarray_i[1], nullptr);
   REQUIRE(rc == TILEDB_OK);
+  rc = tiledb_query_set_subarray_t(ctx_, query, subarray);
+  CHECK(rc == TILEDB_OK);
   rc = tiledb_query_submit(ctx_, query);
   REQUIRE(rc == TILEDB_OK);
 
@@ -654,6 +676,7 @@ void SparseHeterFx::check_read_sparse_array_float_int64(
   // Clean up
   tiledb_array_free(&array);
   tiledb_query_free(&query);
+  tiledb_subarray_free(&subarray);
 }
 
 void SparseHeterFx::check_read_sparse_array_int64_float(
@@ -690,12 +713,17 @@ void SparseHeterFx::check_read_sparse_array_int64_float(
   REQUIRE(rc == TILEDB_OK);
   rc = tiledb_query_set_layout(ctx_, query, layout);
   REQUIRE(rc == TILEDB_OK);
-  rc = tiledb_query_add_range(
-      ctx_, query, 0, &subarray_i[0], &subarray_i[1], nullptr);
+  tiledb_subarray_t* subarray;
+  rc = tiledb_subarray_alloc(ctx_, array, &subarray);
+  CHECK(rc == TILEDB_OK);
+  rc = tiledb_subarray_add_range(
+      ctx_, subarray, 0, &subarray_i[0], &subarray_i[1], nullptr);
   REQUIRE(rc == TILEDB_OK);
-  rc = tiledb_query_add_range(
-      ctx_, query, 1, &subarray_f[0], &subarray_f[1], nullptr);
+  rc = tiledb_subarray_add_range(
+      ctx_, subarray, 1, &subarray_f[0], &subarray_f[1], nullptr);
   REQUIRE(rc == TILEDB_OK);
+  rc = tiledb_query_set_subarray_t(ctx_, query, subarray);
+  CHECK(rc == TILEDB_OK);
   rc = tiledb_query_submit(ctx_, query);
   REQUIRE(rc == TILEDB_OK);
 
@@ -718,26 +746,14 @@ void SparseHeterFx::check_read_sparse_array_int64_float(
   // Clean up
   tiledb_array_free(&array);
   tiledb_query_free(&query);
+  tiledb_subarray_free(&subarray);
 }
 
 TEST_CASE_METHOD(
     SparseHeterFx,
     "C API: Test sparse array with heterogeneous domains (float, int64)",
-    "[capi][sparse][heter][float-int64]") {
-  SECTION("- No serialization") {
-    serialize_ = false;
-  }
-  SECTION("- Serialization") {
-#ifdef TILEDB_SERIALIZATION
-    serialize_ = true;
-    refactored_query_v2_ = GENERATE(true, false);
-#endif
-  }
-
-  SupportedFsLocal local_fs;
-  std::string array_name =
-      local_fs.file_prefix() + local_fs.temp_dir() + "sparse_array_heter";
-  create_temp_dir(local_fs.file_prefix() + local_fs.temp_dir());
+    "[capi][sparse][heter][float-int64][non-rest]") {
+  std::string array_name = prefix_ + "sparse_array_heter";
 
   // Create array
   float dom_f[] = {1.0f, 20.0f};
@@ -987,7 +1003,15 @@ TEST_CASE_METHOD(
   REQUIRE(error == nullptr);
 
   auto rc = tiledb_config_set(
-      config, "sm.consolidation.total_buffer_size", "1048576", &error);
+      config, "sm.mem.consolidation.buffers_weight", "1", &error);
+  REQUIRE(rc == TILEDB_OK);
+  REQUIRE(error == nullptr);
+  rc = tiledb_config_set(
+      config, "sm.mem.consolidation.reader_weight", "5000", &error);
+  REQUIRE(rc == TILEDB_OK);
+  REQUIRE(error == nullptr);
+  rc = tiledb_config_set(
+      config, "sm.mem.consolidation.writer_weight", "5000", &error);
   REQUIRE(rc == TILEDB_OK);
   REQUIRE(error == nullptr);
 
@@ -1085,26 +1109,13 @@ TEST_CASE_METHOD(
       buff_d1_r,
       buff_d2_r,
       buff_a_r);
-
-  remove_temp_dir(local_fs.file_prefix() + local_fs.temp_dir());
 }
 
 TEST_CASE_METHOD(
     SparseHeterFx,
     "C API: Test sparse array with heterogeneous domains (int64, float)",
-    "[capi][sparse][heter][int64-float]") {
-  SECTION("- No serialization") {
-    serialize_ = false;
-  }
-  SECTION("- Serialization") {
-    serialize_ = true;
-    refactored_query_v2_ = GENERATE(true, false);
-  }
-
-  SupportedFsLocal local_fs;
-  std::string array_name =
-      local_fs.file_prefix() + local_fs.temp_dir() + "sparse_array_heter";
-  create_temp_dir(local_fs.file_prefix() + local_fs.temp_dir());
+    "[capi][sparse][heter][int64-float][non-rest]") {
+  std::string array_name = prefix_ + "sparse_array_heter";
 
   // Create array
   float dom_f[] = {1.0f, 20.0f};
@@ -1353,8 +1364,16 @@ TEST_CASE_METHOD(
   REQUIRE(tiledb_config_alloc(&config, &error) == TILEDB_OK);
   REQUIRE(error == nullptr);
 
-  int rc = tiledb_config_set(
-      config, "sm.consolidation.total_buffer_size", "1048576", &error);
+  auto rc = tiledb_config_set(
+      config, "sm.mem.consolidation.buffers_weight", "1", &error);
+  REQUIRE(rc == TILEDB_OK);
+  REQUIRE(error == nullptr);
+  rc = tiledb_config_set(
+      config, "sm.mem.consolidation.reader_weight", "5000", &error);
+  REQUIRE(rc == TILEDB_OK);
+  REQUIRE(error == nullptr);
+  rc = tiledb_config_set(
+      config, "sm.mem.consolidation.writer_weight", "5000", &error);
   REQUIRE(rc == TILEDB_OK);
   REQUIRE(error == nullptr);
 
@@ -1452,6 +1471,4 @@ TEST_CASE_METHOD(
       buff_d1_r,
       buff_d2_r,
       buff_a_r);
-
-  remove_temp_dir(local_fs.file_prefix() + local_fs.temp_dir());
 }
