@@ -56,6 +56,7 @@
 #include "tiledb/sm/misc/tile_overlap.h"
 #include "tiledb/sm/serialization/array.h"
 #include "tiledb/sm/serialization/query.h"
+#include "tiledb/storage_format/uri/generate_uri.h"
 
 int setenv_local(const char* __name, const char* __value) {
 #ifdef _WIN32
@@ -1180,16 +1181,24 @@ void read_array(
   rc = tiledb_query_set_layout(ctx, query, layout);
   CHECK(rc == TILEDB_OK);
 
+  // Create subarray
+  tiledb_subarray_t* subarray;
+  rc = tiledb_subarray_alloc(ctx, array, &subarray);
+  CHECK(rc == TILEDB_OK);
+
   auto dim_num = (unsigned)ranges.size();
   for (unsigned i = 0; i < dim_num; ++i) {
     auto dim_range_num = ranges[i].size() / 2;
     for (size_t j = 0; j < dim_range_num; ++j) {
-      rc = tiledb_query_add_range(
-          ctx, query, i, &ranges[i][2 * j], &ranges[i][2 * j + 1], nullptr);
+      rc = tiledb_subarray_add_range(
+          ctx, subarray, i, &ranges[i][2 * j], &ranges[i][2 * j + 1], nullptr);
       CHECK(rc == TILEDB_OK);
     }
   }
 
+  // Set the subarray
+  tiledb_query_set_subarray_t(ctx, query, subarray);
+
   // Set buffers
   for (const auto& b : buffers) {
     if (b.second.var_ == nullptr) {  // Fixed-sized
@@ -1230,10 +1239,10 @@ void read_array(
 
   // Clean up
   tiledb_query_free(&query);
+  tiledb_subarray_free(&subarray);
 }
 
-int32_t num_commits(const std::string& array_name) {
-  Context ctx;
+int32_t num_commits(Context ctx, const std::string& array_name) {
   VFS vfs(ctx);
 
   // Get the number of write files in the commit directory
@@ -1241,8 +1250,13 @@ int32_t num_commits(const std::string& array_name) {
   return commits_dir.file_count(sm::constants::write_file_suffix);
 }
 
-int32_t num_fragments(const std::string& array_name) {
+int32_t num_commits(const std::string& array_name) {
   Context ctx;
+
+  return num_commits(ctx, array_name);
+}
+
+int32_t num_fragments(Context ctx, const std::string& array_name) {
   VFS vfs(ctx);
 
   // Get all URIs in the array directory
@@ -1251,6 +1265,12 @@ int32_t num_fragments(const std::string& array_name) {
   return static_cast<uint32_t>(uris.size());
 }
 
+int32_t num_fragments(const std::string& array_name) {
+  Context ctx;
+
+  return num_fragments(ctx, array_name);
+}
+
 std::string random_string(const uint64_t l) {
   static const char char_set[] =
       "0123456789"
@@ -1437,449 +1457,19 @@ int deserialize_array_and_query(
       ctx, c_buff, TILEDB_CAPNP, clientside ? 1 : 0, array_uri, query, &array);
 }
 
-int array_open_wrapper(
-    tiledb_ctx_t* client_ctx,
-    tiledb_query_type_t query_type,
-    bool serialize,
-    tiledb_array_t** open_array) {
-  // Open array
-#ifndef TILEDB_SERIALIZATION
-  return tiledb_array_open(client_ctx, *open_array, query_type);
-#endif
-
-  if (!serialize) {
-    return tiledb_array_open(client_ctx, *open_array, query_type);
+sm::URI generate_fragment_uri(sm::Array* array) {
+  if (array == nullptr) {
+    return sm::URI(tiledb::storage_format::generate_timestamped_name(
+        TILEDB_TIMESTAMP_NOW_MS, constants::format_version));
   }
 
-  // Get the config to use it for creating fresh contexts
-  tiledb_config_t* config;
-  int rc = tiledb_ctx_get_config(client_ctx, &config);
-  REQUIRE(rc == TILEDB_OK);
+  uint64_t timestamp = array->timestamp_end_opened_at();
+  auto write_version = array->array_schema_latest().write_version();
 
-  // this helper only applies to refactored array open, so set it in the config
-  // Note: we actually set use_refactored_array_open_and_query_submit instead of
-  // simple use_refactored_array_open here, because we want array_open_wrapper
-  // to be usable in query_v3 tests that require that flag to be set right
-  // from the beginning for full Array objects to be retrieved on array open.
-  // There are dedicated tests in unit-capi-array.cc that are testing array v2
-  // feature with just setting use_refactored_array_open config variable.
-  tiledb_error_t* error = nullptr;
-  rc = tiledb_config_set(
-      config,
-      "rest.use_refactored_array_open_and_query_submit",
-      "true",
-      &error);
-  REQUIRE(rc == TILEDB_OK);
-  REQUIRE(error == nullptr);
-  REQUIRE(
-      tiledb_array_set_config(client_ctx, *open_array, config) == TILEDB_OK);
-
-  auto serialization_type =
-      (tiledb_serialization_type_t)tiledb::sm::SerializationType::CAPNP;
-
-  // 1. Client: Serialize array_open request to Server. First set the query_type
-  // that will be serialized
-  auto type = static_cast<sm::QueryType>(query_type);
-  (*open_array)->array_->set_query_type(type);
-
-  tiledb_buffer_t* buffer;
-  rc = tiledb_serialize_array_open(
-      client_ctx, *open_array, serialization_type, 1, &buffer);
-  REQUIRE(rc == TILEDB_OK);
-
-  // 2. Server : Receive and deserialize array_open_request
-  // Allocate new context for server
-  tiledb_ctx_t* server_ctx;
-  tiledb_ctx_alloc(config, &server_ctx);
-  tiledb_array_t* deserialized_array_server = nullptr;
-  rc = tiledb_deserialize_array_open(
-      server_ctx, buffer, serialization_type, 0, &deserialized_array_server);
-  REQUIRE(rc == TILEDB_OK);
-  tiledb_buffer_free(&buffer);
-
-  // Check that the original and de-serialized array have the same query type
-  REQUIRE(deserialized_array_server->array_->get_query_type() == type);
-
-  // 3. Server: Open the array the request was received for in the requested
-  // mode
-  // This is needed in test, as the deserialized array has a dummy array_uri of
-  // "deserialized_array" set instead of the original one. The Cloud side
-  // already knows the array URI so it's not a problem in the real life scenario
-  deserialized_array_server->array_->set_array_uri(
-      (*open_array)->array_->array_uri());
-
-  // if we need to add start and end timestamps uncomment this code
-  // rc = tiledb_array_set_open_timestamp_start(server_ctx,
-  // deserialized_array_server, array->array_->timestamp_start());
-  // REQUIRE_SAFE(rc == TILEDB_OK);
-  // rc = tiledb_array_set_open_timestamp_end(server_ctx,
-  // deserialized_array_server, array->array_->timestamp_end()); REQUIRE_SAFE(rc
-  // == TILEDB_OK);
-  rc = tiledb_array_open(server_ctx, deserialized_array_server, query_type);
-  REQUIRE(rc == TILEDB_OK);
-
-  // 4. Server -> Client: Send opened Array (serialize)
-  tiledb_buffer_t* buff;
-  rc = tiledb_serialize_array(
-      server_ctx,
-      deserialized_array_server,
-      (tiledb_serialization_type_t)tiledb::sm::SerializationType::CAPNP,
-      1,
-      &buff);
-  REQUIRE(rc == TILEDB_OK);
-  rc = tiledb_array_close(server_ctx, deserialized_array_server);
-  CHECK(rc == TILEDB_OK);
-  tiledb_array_free(&deserialized_array_server);
-  tiledb_ctx_free(&server_ctx);
-  CHECK(server_ctx == nullptr);
-
-  // 5. Client: Receive and deserialize Array (into deserialized_array_client),
-  // in the same way that rest_client does.
-  auto st = tiledb::sm::serialization::array_deserialize(
-      (*open_array)->array_.get(),
-      tiledb::sm::SerializationType::CAPNP,
-      buff->buffer(),
-      client_ctx->storage_manager(),
-      tiledb::test::create_test_memory_tracker());
-  REQUIRE(st.ok());
-
-  // 6. Server: Close array and clean up
-  tiledb_buffer_free(&buff);
-
-  return rc;
-}
-
-int submit_query_wrapper(
-    const Context& client_ctx,
-    const std::string& array_uri,
-    Query* query,
-    ServerQueryBuffers& buffers,
-    bool serialize_query,
-    bool refactored_query_v2,
-    bool finalize) {
-  auto query_c = query->ptr().get();
-  return submit_query_wrapper(
-      client_ctx.ptr().get(),
-      array_uri,
-      &query_c,
-      buffers,
-      serialize_query,
-      refactored_query_v2,
-      finalize);
-}
-
-int submit_query_wrapper(
-    tiledb_ctx_t* client_ctx,
-    const std::string& array_uri,
-    tiledb_query_t** query,
-    ServerQueryBuffers& buffers,
-    bool serialize,
-    bool refactored_query_v2,
-    bool finalize) {
-  int rc = 0;
-#ifndef TILEDB_SERIALIZATION
-  rc = tiledb_query_submit(client_ctx, *query);
-  if (rc != TILEDB_OK) {
-    return rc;
-  }
-
-  if (finalize) {
-    // Finalize query
-    rc = tiledb_query_finalize(client_ctx, *query);
-  }
-
-  return rc;
-#endif
-
-  if (!serialize) {
-    rc = tiledb_query_submit(client_ctx, *query);
-    if (rc != TILEDB_OK) {
-      return rc;
-    }
-
-    if (finalize) {
-      // Finalize query
-      rc = tiledb_query_finalize(client_ctx, *query);
-    }
-
-    return rc;
-  }
-
-  // Get the config to use it for creating fresh contexts
-  tiledb_config_t* config;
-  rc = tiledb_query_get_config(client_ctx, *query, &config);
-  CHECK(rc == TILEDB_OK);
-
-  // Get the array of that query
-  tiledb_array_t* array;
-  REQUIRE(tiledb_query_get_array(client_ctx, *query, &array) == TILEDB_OK);
-
-  // if refactored query v2 set it in the config
-  if (refactored_query_v2) {
-    tiledb_error_t* error = nullptr;
-    rc = tiledb_config_set(
-        config,
-        "rest.use_refactored_array_open_and_query_submit",
-        "true",
-        &error);
-    REQUIRE(rc == TILEDB_OK);
-    REQUIRE(error == nullptr);
-
-    REQUIRE(tiledb_array_close(client_ctx, array) == TILEDB_OK);
-    REQUIRE(tiledb_array_set_config(client_ctx, array, config) == TILEDB_OK);
-    tiledb_query_type_t query_type;
-    REQUIRE_SAFE(
-        tiledb_query_get_type(client_ctx, *query, &query_type) == TILEDB_OK);
-    REQUIRE(tiledb_array_open(client_ctx, array, query_type) == TILEDB_OK);
-  }
-
-  // Get the query type
-  tiledb_query_type_t query_type;
-  REQUIRE_SAFE(
-      tiledb_query_get_type(client_ctx, *query, &query_type) == TILEDB_OK);
-
-  // 1. Client -> Server : Serialize and send query request
-  tiledb_query_t* server_deser_query;
-  std::vector<uint8_t> serialized;
-
-  rc = serialize_query(client_ctx, *query, &serialized, 1);
-  REQUIRE(rc == TILEDB_OK);
-
-  // 2. Server: Deserialize query request
-  // Allocate new context for server
-  tiledb_ctx_t* server_ctx;
-  tiledb_ctx_alloc(config, &server_ctx);
-  tiledb_array_t* array_server = nullptr;
-  if (refactored_query_v2) {
-    rc = deserialize_array_and_query(
-        server_ctx, serialized, &server_deser_query, array_uri.c_str(), 0);
-    REQUIRE_SAFE(rc == TILEDB_OK);
-  } else {
-    // Open array
-    int rc = tiledb_array_alloc(server_ctx, array_uri.c_str(), &array_server);
-    REQUIRE_SAFE(rc == TILEDB_OK);
-
-    rc = tiledb_array_set_open_timestamp_start(
-        server_ctx, array_server, array->array_->timestamp_start());
-    REQUIRE_SAFE(rc == TILEDB_OK);
-    rc = tiledb_array_set_open_timestamp_end(
-        server_ctx, array_server, array->array_->timestamp_end_opened_at());
-    REQUIRE_SAFE(rc == TILEDB_OK);
-
-    rc = tiledb_array_open(server_ctx, array_server, query_type);
-    REQUIRE_SAFE(rc == TILEDB_OK);
-
-    rc = tiledb_query_alloc(
-        server_ctx, array_server, query_type, &server_deser_query);
-    REQUIRE_SAFE(rc == TILEDB_OK);
-    rc = tiledb::test::deserialize_query(
-        server_ctx, serialized, server_deser_query, 0);
-    REQUIRE_SAFE(rc == TILEDB_OK);
-  }
-
-  // This is a feature of the server, not a bug, quoting from query_from_capnp:
-  // "On reads, just set null pointers with accurate size so that the
-  // server can introspect and allocate properly sized buffers separately."
-  // Empty buffers will naturally break query_submit so to go on in test we
-  // need to allocate here as if we were the server.
-  if (query_type == TILEDB_READ) {
-    allocate_query_buffers_server_side(server_ctx, server_deser_query, buffers);
-  }
-
-  // 3. Server: Submit query WITHOUT re-opening the array, using under the hood
-  // the array found in the deserialized query
-  rc = tiledb_query_submit(server_ctx, server_deser_query);
-  if (rc != TILEDB_OK) {
-    return rc;
-  }
-
-  if (finalize) {
-    tiledb_query_status_t status;
-    rc = tiledb_query_get_status(server_ctx, server_deser_query, &status);
-    CHECK(status == TILEDB_COMPLETED);
-
-    rc = tiledb_query_finalize(server_ctx, server_deser_query);
-    if (rc != TILEDB_OK) {
-      return rc;
-    }
-  }
-
-  // Serialize the new query and "send it over the network" (server-side)
-  // 4. Server -> Client : Send query response
-  std::vector<uint8_t> serialized2;
-  rc = serialize_query(server_ctx, server_deser_query, &serialized2, 0);
-  if (rc != TILEDB_OK) {
-    return rc;
-  }
-
-  if (!refactored_query_v2) {
-    // Close array and clean up
-    rc = tiledb_array_close(server_ctx, array_server);
-    CHECK(rc == TILEDB_OK);
-    tiledb_array_free(&array_server);
-  }
-
-  tiledb_ctx_free(&server_ctx);
-  CHECK(server_ctx == nullptr);
-
-  // 5. Client: Deserialize query
-  rc = deserialize_query(client_ctx, serialized2, *query, 1);
-  REQUIRE(rc == TILEDB_OK);
-
-  // Clean up.
-  tiledb_query_free(&server_deser_query);
-
-  return rc;
-}
-
-int finalize_query_wrapper(
-    const Context& client_ctx,
-    const std::string& array_uri,
-    Query* query,
-    bool serialize_query) {
-  auto query_c = query->ptr().get();
-  return finalize_query_wrapper(
-      client_ctx.ptr().get(), array_uri, &query_c, serialize_query);
-}
-
-int finalize_query_wrapper(
-    tiledb_ctx_t* client_ctx,
-    const std::string& array_uri,
-    tiledb_query_t** query,
-    bool serialize) {
-#ifndef TILEDB_SERIALIZATION
-  return tiledb_query_finalize(client_ctx, *query);
-#endif
-
-  if (!serialize) {
-    return tiledb_query_finalize(client_ctx, *query);
-  }
-
-  // Get the config to use it for creating fresh contexts
-  tiledb_config_t* config;
-  auto rc = tiledb_query_get_config(client_ctx, *query, &config);
-  CHECK(rc == TILEDB_OK);
-
-  // Get the query type
-  tiledb_query_type_t query_type;
-  REQUIRE_SAFE(
-      tiledb_query_get_type(client_ctx, *query, &query_type) == TILEDB_OK);
-
-  // 1. Client -> Server : Send query request
-  std::vector<uint8_t> serialized;
-
-  rc = serialize_query(client_ctx, *query, &serialized, 1);
-  REQUIRE(rc == TILEDB_OK);
-
-  // 2. Server: Deserialize query request
-  // Allocate new context for server
-  tiledb_ctx_t* server_ctx;
-  tiledb_ctx_alloc(config, &server_ctx);
-  tiledb_query_t* server_deser_query;
-  rc = deserialize_array_and_query(
-      server_ctx, serialized, &server_deser_query, array_uri.c_str(), 0);
-
-  // 3. Server: Finalize query
-  rc = tiledb_query_finalize(server_ctx, server_deser_query);
-  if (rc != TILEDB_OK) {
-    return rc;
-  }
-
-  /*
-   * TODO: This is disabled because it fails today as the previous serialization
-   * step before finalize, sets the query buffers to NULL for READ queries, so
-   * the following serialization step will hit an assert because of empty
-   * buffers, as query_finalize is a NOOP for READ queries.
-   *
-   * // Serialize the new query and "send it over the network" (server-side)
-   * 3. Server -> Client : Send query response
-   * std::vector<uint8_t> serialized2;
-   * rc = tiledb_query_v2_serialize(
-   *     ctx, array_uri.c_str(), serialized, false, server_deser_query, query);
-   * REQUIRE_SAFE(rc == TILEDB_OK);
-   */
-
-  // Clean up.
-  tiledb_query_free(&server_deser_query);
-
-  return rc;
-}
-
-void allocate_query_buffers_server_side(
-    tiledb_ctx_t* ctx,
-    tiledb_query_t* query,
-    ServerQueryBuffers& query_buffers) {
-  int rc = 0;
-  auto buffer_names = query->query_->buffer_names();
-  const auto aggregate_names = query->query_->aggregate_buffer_names();
-  buffer_names.insert(
-      buffer_names.end(), aggregate_names.begin(), aggregate_names.end());
-
-  for (uint64_t i = 0; i < buffer_names.size(); i++) {
-    const auto& name = buffer_names[i];
-    const auto& buff = query->query_->buffer(name);
-    const auto& schema = query->query_->array_schema();
-
-    // TODO: This is yet another instance where there needs to be a common
-    // mechanism for reporting the common properties of a field.
-    // Refactor to use query_field_t.
-    bool var_size = false;
-    bool nullable = false;
-    if (query->query_->is_aggregate(name)) {
-      var_size =
-          query->query_->get_aggregate(name).value()->aggregation_var_sized();
-      nullable =
-          query->query_->get_aggregate(name).value()->aggregation_nullable();
-    } else {
-      var_size = schema.var_size(name);
-      nullable = schema.is_nullable(name);
-    }
-
-    if (var_size && buff.buffer_var_ == nullptr) {
-      // Variable-sized buffer
-      query_buffers.attr_or_dim_data.emplace_back(*buff.buffer_var_size_);
-      query_buffers.attr_or_dim_off.emplace_back(
-          (*buff.buffer_size_) / sizeof(sm::constants::cell_var_offset_size));
-      rc = tiledb_query_set_data_buffer(
-          ctx,
-          query,
-          name.c_str(),
-          query_buffers.attr_or_dim_data.back().data(),
-          buff.buffer_var_size_);
-      REQUIRE_SAFE(rc == TILEDB_OK);
-      rc = tiledb_query_set_offsets_buffer(
-          ctx,
-          query,
-          name.c_str(),
-          query_buffers.attr_or_dim_off.back().data(),
-          buff.buffer_size_);
-      REQUIRE_SAFE(rc == TILEDB_OK);
-    }
-
-    if (name == TILEDB_COORDS || (!var_size && buff.buffer_ == nullptr)) {
-      // Fixed-length buffer or Coords
-      query_buffers.attr_or_dim.emplace_back(*buff.buffer_size_);
-      rc = tiledb_query_set_data_buffer(
-          ctx,
-          query,
-          name.c_str(),
-          query_buffers.attr_or_dim.back().data(),
-          buff.buffer_size_);
-      REQUIRE_SAFE(rc == TILEDB_OK);
-    }
-
-    if (nullable) {
-      // nullable
-      query_buffers.attr_or_dim_nullable.emplace_back(
-          *buff.validity_vector_.buffer_size());
-      rc = tiledb_query_set_validity_buffer(
-          ctx,
-          query,
-          name.c_str(),
-          query_buffers.attr_or_dim_nullable.back().data(),
-          buff.validity_vector_.buffer_size());
-    }
-  }
+  auto new_fragment_str = tiledb::storage_format::generate_timestamped_name(
+      timestamp, write_version);
+  auto frag_dir_uri = array->array_directory().get_fragments_dir(write_version);
+  return frag_dir_uri.join_path(new_fragment_str);
 }
 
 template void check_subarray<int8_t>(
