@@ -33,7 +33,6 @@
 #ifndef TILEDB_TEST_HELPERS_H
 #define TILEDB_TEST_HELPERS_H
 
-#include <tiledb/common/logger_public.h>
 #include "test/support/src/coords_workaround.h"
 #include "test/support/src/mem_helpers.h"
 #include "tiledb.h"
@@ -762,12 +761,24 @@ void read_array(
     tiledb_layout_t layout,
     const QueryBuffers& buffers);
 
+/**
+ * Returns the number of commits in the input array,
+ * given a context.
+ */
+int32_t num_commits(Context ctx, const std::string& array_name);
+
 /**
  * Returns the number of commits in the input array,
  * appropriately excluding special files and subdirectories.
  */
 int32_t num_commits(const std::string& array_name);
 
+/**
+ * Returns the number of fragments in the input array,
+ * given a context.
+ */
+int32_t num_fragments(Context ctx, const std::string& array_name);
+
 /**
  * Returns the number of fragments in the input array,
  * appropriately excluding special files and subdirectories.
@@ -907,98 +918,17 @@ int deserialize_array_and_query(
     bool clientside);
 
 /**
- * Helper method that wraps tiledb_array_open() and inserts a serialization
- * step, if serialization is enabled. This wrapper models exclusively the
- * refactored array open (array open v2) serialization path. The added
- * serialization steps are designed to closely mimic the behavior of the REST
- * server.
+ * Helper function that generates a fragment uri for an array.
+ * If the `array` arg is specified, the full URI is generated using
+ * the write version of the array schema and the timestamp
+ * the array was opened at. If unspecified, the function
+ * will generate only the fragment name using the current timestamp
+ * and the latest library format version.
  *
- * @param ctx Context.
- * @param query_type Type of query to open the array for.
- * @param serialize True if this is a remote array open, false if not.
- * @param open_array Output open array.
+ * @param array A TileDB array
+ * @return A test fragment uri
  */
-int array_open_wrapper(
-    tiledb_ctx_t* ctx,
-    tiledb_query_type_t query_type,
-    bool serialize,
-    tiledb_array_t** open_array);
-
-/**
- * C API
- * Helper method that wraps tiledb_query_submit() and inserts serialization
- * steps, if serialization is enabled. The added serialization steps are
- * designed to closely mimic the behavior of the REST server.
- *
- * @param ctx Context.
- * @param array_uri URI of the array the query submit is for.
- * @param query Input/Output tiledb query object to submit/get back.
- * @param buffers Stack allocated buffers to be used by the step simulating
- * "server side buffer allocation" for READ queries. They can only be allocated
- * e.g. once in the test Fixture definition or just before the call to
- * submit_query_wrapper.
- * @param serialize_query True if this is a remote array open, false if not.
- * @param refactored_query_v2 If
- * "rest.use_refactored_array_open_and_query_submit" should be true.
- * @param finalize Finalize or not the query after submitting it.
- */
-int submit_query_wrapper(
-    tiledb_ctx_t* ctx,
-    const std::string& array_uri,
-    tiledb_query_t** query,
-    ServerQueryBuffers& buffers,
-    bool serialize_query,
-    bool refactored_query_v2 = false,
-    bool finalize = true);
-
-/** C++ wrapper of submit_query_wrapper */
-int submit_query_wrapper(
-    const Context& ctx,
-    const std::string& array_uri,
-    Query* query,
-    ServerQueryBuffers& buffers,
-    bool serialize_query,
-    bool refactored_query_v2 = false,
-    bool finalize = true);
-
-/**
- * C API
- * Helper method that wraps tiledb_query_finalize() and inserts serialization
- * steps, if serialization is enabled. The added serialization steps are
- * designed to closely mimic the behavior of the REST server.
- *
- * @param ctx Context.
- * @param array_uri URI of the array the query finalize is for.
- * @param query Input/Output tiledb query object to submit/get back.
- * @param serialize_query True if this is a remote array open, false if not.
- */
-int finalize_query_wrapper(
-    const Context& ctx,
-    const std::string& array_uri,
-    Query* query,
-    bool serialize_query);
-
-/** C++ wrapper of finalize_query_wrapper */
-int finalize_query_wrapper(
-    tiledb_ctx_t* ctx,
-    const std::string& array_uri,
-    tiledb_query_t** query,
-    bool serialize_query);
-
-/**
- * Helper function that allocates buffers on a query object that has been
- * deserialized on the "server" side. This is a necassary step for serialized
- * READ queries.
- *
- * @param ctx Context.
- * @param query Query for which we are allocating the buffers.
- * @param query_buffers Input/Output allocated buffers.
- */
-void allocate_query_buffers_server_side(
-    tiledb_ctx_t* ctx,
-    tiledb_query_t* query,
-    ServerQueryBuffers& query_buffers);
-
+sm::URI generate_fragment_uri(sm::Array* array);
 }  // namespace tiledb::test
 
 #endif
