@@ -55,6 +55,8 @@ class ArraySchema;
 class SchemaEvolution;
 class FragmentMetadata;
 class MemoryTracker;
+class UpdateValue;
+class QueryCondition;
 enum class QueryType : uint8_t;
 
 /**
@@ -99,7 +101,8 @@ class OpenedArray {
       uint64_t timestamp_start,
       uint64_t timestamp_end_opened_at,
       bool is_remote)
-      : array_dir_(ArrayDirectory(resources, array_uri))
+      : resources_(resources)
+      , array_dir_(ArrayDirectory(resources, array_uri))
       , array_schema_latest_(nullptr)
       , metadata_(memory_tracker)
       , metadata_loaded_(false)
@@ -207,7 +210,21 @@ class OpenedArray {
     return is_remote_;
   }
 
+  /**
+   * Loads the delete and update conditions from storage.
+   *
+   * @return Status, vector of the conditions, vector of the update values.
+   */
+  tuple<
+      Status,
+      optional<std::vector<QueryCondition>>,
+      optional<std::vector<std::vector<UpdateValue>>>>
+  load_delete_and_update_conditions();
+
  private:
+  /** The context resources. */
+  ContextResources& resources_;
+
   /** The array directory object for listing URIs. */
   optional<ArrayDirectory> array_dir_;
 
@@ -420,26 +437,78 @@ class Array {
   Status close();
 
   /**
-   * Deletes the Array data with given URI.
+   * Performs deletion of local fragments with the given parent URI,
+   * between the provided timestamps.
    *
-   * @param uri The uri of the Array whose data is to be deleted.
+   * @param resources The context resources.
+   * @param uri The uri of the Array whose fragments are to be deleted.
+   * @param timestamp_start The start timestamp at which to delete fragments.
+   * @param timestamp_end The end timestamp at which to delete fragments.
+   * @param array_dir An optional ArrayDirectory from which to delete fragments.
    *
-   * @pre The Array must be open for exclusive writes
+   * @section Maturity Notes
+   * This is legacy code, ported from StorageManager during its removal process.
+   * Its existence supports the non-static `delete_fragments` API below,
+   * performing the actual deletion of fragments. This function is slated for
+   * removal and should be directly integrated into the function below.
    */
-  void delete_array(const URI& uri);
+  static void delete_fragments(
+      ContextResources& resources,
+      const URI& uri,
+      uint64_t timestamp_start,
+      uint64_t timstamp_end,
+      std::optional<ArrayDirectory> array_dir = std::nullopt);
 
   /**
-   * Deletes the fragments from the Array with given URI.
+   * Handles local and remote deletion of fragments between the provided
+   * timestamps from an open array with the given URI.
    *
    * @param uri The uri of the Array whose fragments are to be deleted.
    * @param timestamp_start The start timestamp at which to delete fragments.
    * @param timestamp_end The end timestamp at which to delete fragments.
    *
    * @pre The Array must be open for exclusive writes
+   *
+   * @section Maturity Notes
+   * This API is an interim version of its final product, awaiting rewrite.
+   * As is, it handles the incoming URI and invokes the remote or local function
+   * call accordingly. The local, static function above is legacy code which
+   * exists only to support this function. A rewrite should integrate the two
+   * and remove the need for any static APIs.
    */
   void delete_fragments(
       const URI& uri, uint64_t timestamp_start, uint64_t timstamp_end);
 
+  /**
+   * Performs deletion of the local array with the given parent URI.
+   *
+   * @param resources The context resources.
+   * @param uri The uri of the Array whose data is to be deleted.
+   *
+   * @section Maturity Notes
+   * This is legacy code, ported from StorageManager during its removal process.
+   * Its existence supports the non-static `delete_array` API below,
+   * performing the actual deletion of array data. This function is slated for
+   * removal and should be directly integrated into the function below.
+   */
+  static void delete_array(ContextResources& resources, const URI& uri);
+
+  /**
+   * Handles local and remote deletion of an open array with the given URI.
+   *
+   * @param uri The uri of the Array whose data is to be deleted.
+   *
+   * @pre The Array must be open for exclusive writes
+   *
+   * @section Maturity Notes
+   * This API is an interim version of its final product, awaiting rewrite.
+   * As is, it handles the incoming URI and invokes the remote or local function
+   * call accordingly. The local, static function above is legacy code which
+   * exists only to support this function. A rewrite should integrate the two
+   * and remove the need for any static APIs.
+   */
+  void delete_array(const URI& uri);
+
   /**
    * Deletes the fragments with the given URIs from the Array with given URI.
    *
@@ -468,6 +537,19 @@ class Array {
     opened_array_->fragment_metadata() = fragment_metadata;
   }
 
+  /**
+   * Retrieves the encryption type.
+   *
+   * @param resources The context resources.
+   * @param uri The URI of the array.
+   * @param encryption_type Set to the encryption type.
+   * @return Status
+   */
+  static Status encryption_type(
+      ContextResources& resources,
+      const URI& uri,
+      EncryptionType* encryption_type);
+
   /**
    * Get the enumeration for the given name.
    *
@@ -754,10 +836,106 @@ class Array {
 
   /**
    * Returns the non-empty domain of the opened array. If the non_empty_domain
-   * has not been computed or loaded it will be loaded first
+   * has not been computed or loaded it will be loaded first.
    */
   const NDRange non_empty_domain();
 
+  /**
+   * Retrieves the non-empty domain from the opened array.
+   * This is the union of the non-empty domains of the array fragments.
+   *
+   * @param domain The domain to be retrieved.
+   * @param is_empty `true` if the non-empty domain (and array) is empty.
+   */
+  void non_empty_domain(NDRange* domain, bool* is_empty);
+
+  /**
+   * Retrieves the non-empty domain from the opened array.
+   * This is the union of the non-empty domains of the array fragments.
+   *
+   * @pre The array dimensions are numeric and of the same type.
+   *
+   * @param domain The domain to be retrieved.
+   * @param is_empty `true` if the non-empty domain (and array) is empty.
+   */
+  void non_empty_domain(void* domain, bool* is_empty);
+
+  /**
+   * Returns the non-empty domain of the opened array on the given dimension.
+   * This is the union of the non-empty domains of the array fragments.
+   *
+   * @param idx The dimension index.
+   * @param domain The domain to be retrieved.
+   * @param is_empty `true` if the non-empty domain (and array) is empty.
+   */
+  void non_empty_domain_from_index(unsigned idx, void* domain, bool* is_empty);
+
+  /**
+   * Returns the non-empty domain size of the opened array on the given
+   * dimension. This is the union of the non-empty domains of the array
+   * fragments. Applicable only to var-sized dimensions.
+   *
+   * @param idx The dimension index.
+   * @param start_size The size in bytes of the range start.
+   * @param end_size The size in bytes of the range end.
+   * @param is_empty `true` if the non-empty domain (and array) is empty.
+   */
+  void non_empty_domain_var_size_from_index(
+      unsigned idx, uint64_t* start_size, uint64_t* end_size, bool* is_empty);
+
+  /**
+   * Returns the non-empty domain of the opened array on the given dimension.
+   * This is the union of the non-empty domains of the array fragments.
+   * Applicable only to var-sized dimensions.
+   *
+   * @param idx The dimension index.
+   * @param start The domain range start to set.
+   * @param end The domain range end to set.
+   * @param is_empty `true` if the non-empty domain (and array) is empty.
+   */
+  void non_empty_domain_var_from_index(
+      unsigned idx, void* start, void* end, bool* is_empty);
+
+  /**
+   * Returns the non-empty domain of the opened array on the given dimension.
+   * This is the union of the non-empty domains of the array fragments.
+   *
+   * @param field_name The dimension name.
+   * @param domain The domain to be retrieved.
+   * @param is_empty `true` if the non-empty domain (and array) is empty.
+   */
+  void non_empty_domain_from_name(
+      std::string_view field_name, void* domain, bool* is_empty);
+
+  /**
+   * Returns the non-empty domain size of the open array on the given dimension.
+   * This is the union of the non-empty domains of the array fragments.
+   * Applicable only to var-sized dimensions.
+   *
+   * @param field_name The dimension name.
+   * @param start_size The size in bytes of the range start.
+   * @param end_size The size in bytes of the range end.
+   * @param is_empty `true` if the non-empty domain (and array) is empty.
+   */
+  void non_empty_domain_var_size_from_name(
+      std::string_view field_name,
+      uint64_t* start_size,
+      uint64_t* end_size,
+      bool* is_empty);
+
+  /**
+   * Returns the non-empty domain of the opened array on the given dimension.
+   * This is the union of the non-empty domains of the array fragments.
+   * Applicable only to var-sized dimensions.
+
+   * @param field_name The dimension name.
+   * @param start The domain range start to set.
+   * @param end The domain range end to set.
+   * @param is_empty `true` if the non-empty domain (and array) is empty.
+   */
+  void non_empty_domain_var_from_name(
+      std::string_view field_name, void* start, void* end, bool* is_empty);
+
   /**
    * Retrieves the array metadata object that is already loaded. If it's not yet
    * loaded it will be empty.
