@@ -43,10 +43,10 @@
  */
 #include "tiledb/api/c_api/attribute/attribute_api_external_experimental.h"
 #include "tiledb/api/c_api/enumeration/enumeration_api_experimental.h"
-#include "tiledb/api/c_api/group/group_api_external_experimental.h"
 #include "tiledb/api/c_api/query_aggregate/query_aggregate_api_external_experimental.h"
 #include "tiledb/api/c_api/query_field/query_field_api_external_experimental.h"
 #include "tiledb/api/c_api/query_plan/query_plan_api_external_experimental.h"
+#include "tiledb/api/c_api/vfs/vfs_api_experimental.h"
 #include "tiledb_dimension_label_experimental.h"
 
 /* ********************************* */
@@ -358,41 +358,6 @@ TILEDB_EXPORT int32_t tiledb_array_schema_add_enumeration(
 /*               ARRAY               */
 /* ********************************* */
 
-/**
- * Deletes all written array data.
- *
- * **Example:**
- *
- * @code{.c}
- * tiledb_array_delete(ctx, "hdfs:///temp/my_array");
- * @endcode
- *
- * @param ctx The TileDB context.
- * @param uri The Array's URI.
- * @return `TILEDB_OK` for success and `TILEDB_ERR` for error.
- */
-TILEDB_EXPORT int32_t tiledb_array_delete(tiledb_ctx_t* ctx, const char* uri)
-    TILEDB_NOEXCEPT;
-
-/**
- * Note: This API is deprecated and replaced with tiledb_array_delete (above).
- *
- * Deletes all written array data.
- *
- * **Example:**
- *
- * @code{.c}
- * tiledb_array_delete_array(ctx, array, "hdfs:///temp/my_array");
- * @endcode
- *
- * @param ctx The TileDB context.
- * @param array The array to delete the data from.
- * @param uri The Array's URI.
- * @return `TILEDB_OK` for success and `TILEDB_ERR` for error.
- */
-TILEDB_DEPRECATED_EXPORT int32_t tiledb_array_delete_array(
-    tiledb_ctx_t* ctx, tiledb_array_t* array, const char* uri) TILEDB_NOEXCEPT;
-
 /**
  * Evolve array schema of an array.
  *
@@ -457,27 +422,6 @@ TILEDB_EXPORT capi_return_t tiledb_array_get_enumeration(
 TILEDB_EXPORT capi_return_t tiledb_array_load_all_enumerations(
     tiledb_ctx_t* ctx, const tiledb_array_t* array) TILEDB_NOEXCEPT;
 
-/**
- * Upgrades an array to the latest format version.
- *
- * **Example:**
- *
- * @code{.c}
- * const char* array_uri="test_array";
- * tiledb_array_upgrade_version(ctx, array_uri);
- * @endcode
- *
- * @param ctx The TileDB context.
- * @param array_uri The uri of the array.
- * @param config Configuration parameters for the upgrade
- *     (`nullptr` means default, which will use the config from `ctx`).
- * @return `TILEDB_OK` for success and `TILEDB_ERR` for error.
- */
-TILEDB_EXPORT int32_t tiledb_array_upgrade_version(
-    tiledb_ctx_t* ctx,
-    const char* array_uri,
-    tiledb_config_t* config) TILEDB_NOEXCEPT;
-
 /* ********************************* */
 /*               QUERY               */
 /* ********************************* */
@@ -518,6 +462,7 @@ TILEDB_EXPORT int32_t tiledb_subarray_add_point_ranges(
     const void* start,
     uint64_t count) TILEDB_NOEXCEPT;
 
+#ifndef TILEDB_REMOVE_DEPRECATIONS
 /**
  * Adds a set of point ranges along subarray dimension index. Each value
  * in the target array is added as `add_range(x,x)` for count elements.
@@ -548,6 +493,7 @@ TILEDB_DEPRECATED_EXPORT int32_t tiledb_query_add_point_ranges(
     uint32_t dim_idx,
     const void* start,
     uint64_t count) TILEDB_NOEXCEPT;
+#endif  // TILEDB_REMOVE_DEPRECATIONS
 
 /**
  * Get the number of relevant fragments from the subarray. Should only be
@@ -745,7 +691,8 @@ TILEDB_EXPORT capi_return_t tiledb_ctx_alloc_with_error(
  * @param[in] ctx The TileDB context.
  * @param[in] array_uri The name of the TileDB array whose metadata will
  *     be consolidated.
- * @param[in] fragment_uris URIs of the fragments to consolidate.
+ * @param[in] fragment_uris Fragment names of the fragments to consolidate. The
+ *     names can be recovered using tiledb_fragment_info_get_fragment_name.
  * @param[in] num_fragments Number of URIs to consolidate.
  * @param config Configuration parameters for the consolidation
  *     (`nullptr` means default, which will use the config from \p ctx).
