@@ -89,7 +89,7 @@ int32_t tiledb_filestore_schema_create(
         &context.resources().stats(),
         context.compute_tp(),
         context.io_tp(),
-        context.storage_manager()->config());
+        context.resources().config());
     uint64_t file_size;
     throw_if_not_ok(vfs.file_size(tiledb::sm::URI(uri), &file_size));
     if (file_size) {
@@ -202,7 +202,7 @@ int32_t tiledb_filestore_uri_import(
       &context.resources().stats(),
       context.compute_tp(),
       context.io_tp(),
-      context.storage_manager()->config());
+      context.resources().config());
   uint64_t file_size;
   throw_if_not_ok(vfs.file_size(tiledb::sm::URI(file_uri), &file_size));
   if (!file_size) {
@@ -271,8 +271,8 @@ int32_t tiledb_filestore_uri_import(
   // timestamped fragments in row-major order.
   bool is_tiledb_uri = array->is_remote();
   uint64_t tile_extent = compute_tile_extent_based_on_file_size(file_size);
-  auto buffer_size = get_buffer_size_from_config(
-      context.storage_manager()->config(), tile_extent);
+  auto buffer_size =
+      get_buffer_size_from_config(context.resources().config(), tile_extent);
 
   tiledb::sm::Query query(context.storage_manager(), array);
   throw_if_not_ok(query.set_layout(tiledb::sm::Layout::GLOBAL_ORDER));
@@ -296,7 +296,7 @@ int32_t tiledb_filestore_uri_import(
   uint64_t range_arr[] = {static_cast<uint64_t>(0), last_space_tile_boundary};
   tiledb::type::Range subarray_range(
       static_cast<void*>(range_arr), sizeof(uint64_t) * 2);
-  throw_if_not_ok(subarray.add_range(0, std::move(subarray_range)));
+  subarray.add_range(0, std::move(subarray_range));
   query.set_subarray(subarray);
 
   auto tiledb_cloud_fix = [&](uint64_t start, uint64_t end) {
@@ -312,8 +312,7 @@ int32_t tiledb_filestore_uri_import(
     uint64_t cloud_fix_range_arr[] = {start, end};
     tiledb::type::Range subarray_range_cloud_fix(
         static_cast<void*>(cloud_fix_range_arr), sizeof(uint64_t) * 2);
-    throw_if_not_ok(
-        subarray_cloud_fix.add_range(0, std::move(subarray_range_cloud_fix)));
+    subarray_cloud_fix.add_range(0, std::move(subarray_range_cloud_fix));
     query.set_subarray(subarray_cloud_fix);
     uint64_t data_buff_len = end - start + 1;
     throw_if_not_ok(query.set_data_buffer(
@@ -391,7 +390,7 @@ int32_t tiledb_filestore_uri_export(
       &context.resources().stats(),
       context.compute_tp(),
       context.io_tp(),
-      context.storage_manager()->config());
+      context.resources().config());
   if (!vfs.open_file(tiledb::sm::URI(file_uri), tiledb::sm::VFSMode::VFS_WRITE)
            .ok()) {
     throw api::CAPIException(
@@ -423,8 +422,8 @@ int32_t tiledb_filestore_uri_export(
 
   uint64_t file_size = *static_cast<const uint64_t*>(file_size_ptr);
   uint64_t tile_extent = compute_tile_extent_based_on_file_size(file_size);
-  auto buffer_size = get_buffer_size_from_config(
-      context.storage_manager()->config(), tile_extent);
+  auto buffer_size =
+      get_buffer_size_from_config(context.resources().config(), tile_extent);
 
   std::vector<std::byte> data(buffer_size);
   uint64_t start_range = 0;
@@ -440,7 +439,7 @@ int32_t tiledb_filestore_uri_export(
     uint64_t subarray_range_arr[] = {start_range, end_range};
     tiledb::type::Range subarray_range(
         static_cast<void*>(subarray_range_arr), sizeof(uint64_t) * 2);
-    throw_if_not_ok(subarray.add_range(0, std::move(subarray_range)));
+    subarray.add_range(0, std::move(subarray_range));
 
     tiledb::sm::Query query(context.storage_manager(), array);
     throw_if_not_ok(query.set_layout(tiledb::sm::Layout::ROW_MAJOR));
@@ -558,7 +557,7 @@ int32_t tiledb_filestore_buffer_import(
       static_cast<uint64_t>(0), static_cast<uint64_t>(size - 1)};
   tiledb::type::Range subarray_range(
       static_cast<void*>(subarray_range_arr), sizeof(uint64_t) * 2);
-  throw_if_not_ok(subarray.add_range(0, std::move(subarray_range)));
+  subarray.add_range(0, std::move(subarray_range));
 
   query.set_subarray(subarray);
   uint64_t size_tmp = size;
@@ -620,7 +619,7 @@ int32_t tiledb_filestore_buffer_export(
       static_cast<uint64_t>(offset), static_cast<uint64_t>(offset + size - 1)};
   tiledb::type::Range subarray_range(
       static_cast<void*>(subarray_range_arr), sizeof(uint64_t) * 2);
-  throw_if_not_ok(subarray.add_range(0, std::move(subarray_range)));
+  subarray.add_range(0, std::move(subarray_range));
 
   tiledb::sm::Query query(context.storage_manager(), array);
   throw_if_not_ok(query.set_layout(tiledb::sm::Layout::ROW_MAJOR));
