@@ -57,6 +57,18 @@ using tiledb::common::filesystem::directory_entry;
 namespace tiledb {
 namespace sm {
 
+/** Converts an Azure nullable value to an STL optional. */
+template <class T>
+std::optional<T> from_azure_nullable(const ::Azure::Nullable<T>& value) {
+  return value ? std::optional(*value) : nullopt;
+}
+
+/** Converts an STL optional value to an Azure nullable. */
+template <class T>
+::Azure::Nullable<T> to_azure_nullable(const std::optional<T>& value) {
+  return value ? *value : ::Azure::Nullable<T>{};
+}
+
 /* ********************************* */
 /*     CONSTRUCTORS & DESTRUCTORS    */
 /* ********************************* */
@@ -79,7 +91,7 @@ std::string get_config_with_env_fallback(
   return result;
 }
 
-std::string get_blob_endpoint(
+std::optional<std::string> get_blob_endpoint(
     const Config& config, const std::string& account_name) {
   std::string sas_token = get_config_with_env_fallback(
       config, "vfs.azure.storage_sas_token", "AZURE_STORAGE_SAS_TOKEN");
@@ -90,9 +102,7 @@ std::string get_blob_endpoint(
     if (!account_name.empty()) {
       result = "https://" + account_name + ".blob.core.windows.net";
     } else {
-      LOG_WARN(
-          "Neither the 'vfs.azure.storage_account_name' nor the "
-          "'vfs.azure.blob_endpoint' options are specified.");
+      return std::nullopt;
     }
   } else if (!(utils::parse::starts_with(result, "http://") ||
                utils::parse::starts_with(result, "https://"))) {
@@ -124,7 +134,20 @@ static bool has_sas_token(const Config& config) {
   return !sas_token.empty();
 }
 
-AzureParameters::AzureParameters(const Config& config)
+std::optional<AzureParameters> AzureParameters::create(const Config& config) {
+  auto account_name = get_config_with_env_fallback(
+      config, "vfs.azure.storage_account_name", "AZURE_STORAGE_ACCOUNT");
+  auto blob_endpoint = get_blob_endpoint(config, account_name);
+  if (!blob_endpoint) {
+    return std::nullopt;
+  }
+  return AzureParameters{config, account_name, *blob_endpoint};
+}
+
+AzureParameters::AzureParameters(
+    const Config& config,
+    const std::string& account_name,
+    const std::string& blob_endpoint)
     : max_parallel_ops_(
           config.get<uint64_t>("vfs.azure.max_parallel_ops", Config::must_find))
     , block_list_block_size_(config.get<uint64_t>(
@@ -138,11 +161,10 @@ AzureParameters::AzureParameters(const Config& config)
           "vfs.azure.max_retry_delay_ms", Config::must_find)))
     , use_block_list_upload_(config.get<bool>(
           "vfs.azure.use_block_list_upload", Config::must_find))
-    , account_name_(get_config_with_env_fallback(
-          config, "vfs.azure.storage_account_name", "AZURE_STORAGE_ACCOUNT"))
+    , account_name_(account_name)
     , account_key_(get_config_with_env_fallback(
           config, "vfs.azure.storage_account_key", "AZURE_STORAGE_KEY"))
-    , blob_endpoint_(get_blob_endpoint(config, account_name_))
+    , blob_endpoint_(blob_endpoint)
     , ssl_cfg_(config)
     , has_sas_token_(has_sas_token(config)) {
 }
@@ -153,7 +175,7 @@ Status Azure::init(const Config& config, ThreadPool* const thread_pool) {
         Status_AzureError("Can't initialize with null thread pool."));
   }
   thread_pool_ = thread_pool;
-  azure_params_ = config;
+  azure_params_ = AzureParameters::create(config);
   return Status::Ok();
 }
 
@@ -200,14 +222,22 @@ Azure::AzureClientSingleton::get(const AzureParameters& params) {
   // Construct the Azure SDK blob service client.
   // We pass a shared key if it was specified.
   if (!params.account_key_.empty()) {
-    client_ =
-        tdb_unique_ptr<::Azure::Storage::Blobs::BlobServiceClient>(tdb_new(
-            ::Azure::Storage::Blobs::BlobServiceClient,
-            params.blob_endpoint_,
-            make_shared<::Azure::Storage::StorageSharedKeyCredential>(
-                HERE(), params.account_name_, params.account_key_),
-            options));
-    return *client_;
+    // If we don't have an account name, warn and try other authentication
+    // methods.
+    if (params.account_name_.empty()) {
+      LOG_WARN(
+          "Azure storage account name must be set when specifying account key. "
+          "Account key will be ignored.");
+    } else {
+      client_ =
+          tdb_unique_ptr<::Azure::Storage::Blobs::BlobServiceClient>(tdb_new(
+              ::Azure::Storage::Blobs::BlobServiceClient,
+              params.blob_endpoint_,
+              make_shared<::Azure::Storage::StorageSharedKeyCredential>(
+                  HERE(), params.account_name_, params.account_key_),
+              options));
+      return *client_;
+    }
   }
 
   // Otherwise, if we did not specify an SAS token
@@ -524,7 +554,7 @@ Status Azure::is_blob(
   return Status_Ok();
 }
 
-std::string Azure::remove_front_slash(const std::string& path) const {
+std::string Azure::remove_front_slash(const std::string& path) {
   if (path.front() == '/') {
     return path.substr(1, path.length());
   }
@@ -532,7 +562,7 @@ std::string Azure::remove_front_slash(const std::string& path) const {
   return path;
 }
 
-std::string Azure::add_trailing_slash(const std::string& path) const {
+std::string Azure::add_trailing_slash(const std::string& path) {
   if (path.back() != '/') {
     return path + "/";
   }
@@ -540,7 +570,7 @@ std::string Azure::add_trailing_slash(const std::string& path) const {
   return path;
 }
 
-std::string Azure::remove_trailing_slash(const std::string& path) const {
+std::string Azure::remove_trailing_slash(const std::string& path) {
   if (path.back() == '/') {
     return path.substr(0, path.length() - 1);
   }
@@ -1069,6 +1099,69 @@ Status Azure::write_blocks(
   return Status::Ok();
 }
 
+LsObjects Azure::list_blobs_impl(
+    const std::string& container_name,
+    const std::string& blob_path,
+    bool recursive,
+    int max_keys,
+    optional<std::string>& continuation_token) const {
+  try {
+    ::Azure::Storage::Blobs::ListBlobsOptions options{
+        .Prefix = blob_path,
+        .ContinuationToken = to_azure_nullable(continuation_token),
+        .PageSizeHint = max_keys};
+    auto container_client = client().GetBlobContainerClient(container_name);
+    auto to_directory_entry =
+        [&container_name](const ::Azure::Storage::Blobs::Models::BlobItem& item)
+        -> LsObjects::value_type {
+      return {
+          "azure://" + container_name + "/" +
+              remove_front_slash(remove_trailing_slash(item.Name)),
+          item.BlobSize >= 0 ? static_cast<uint64_t>(item.BlobSize) : 0};
+    };
+
+    LsObjects result;
+    if (recursive) {
+      auto response = container_client.ListBlobs(options);
+
+      continuation_token = from_azure_nullable(response.NextPageToken);
+
+      result.reserve(response.Blobs.size());
+      std::transform(
+          response.Blobs.begin(),
+          response.Blobs.end(),
+          std::back_inserter(result),
+          to_directory_entry);
+    } else {
+      auto response = container_client.ListBlobsByHierarchy("/", options);
+
+      continuation_token = from_azure_nullable(response.NextPageToken);
+
+      result.reserve(response.Blobs.size() + response.BlobPrefixes.size());
+      std::transform(
+          response.Blobs.begin(),
+          response.Blobs.end(),
+          std::back_inserter(result),
+          to_directory_entry);
+      std::transform(
+          response.BlobPrefixes.begin(),
+          response.BlobPrefixes.end(),
+          std::back_inserter(result),
+          [&container_name](std::string& name) -> LsObjects::value_type {
+            return {
+                "azure://" + container_name + "/" +
+                    remove_front_slash(add_trailing_slash(name)),
+                0};
+          });
+    }
+
+    return result;
+  } catch (const ::Azure::Storage::StorageException& e) {
+    throw AzureException(
+        "List blobs failed on: " + blob_path + "; " + e.Message);
+  }
+}
+
 Status Azure::upload_block(
     const std::string& container_name,
     const std::string& blob_path,
@@ -1093,7 +1186,7 @@ Status Azure::upload_block(
 Status Azure::parse_azure_uri(
     const URI& uri,
     std::string* const container_name,
-    std::string* const blob_path) const {
+    std::string* const blob_path) {
   assert(uri.is_azure());
   const std::string uri_str = uri.to_string();
 
