@@ -224,6 +224,8 @@ struct S3Parameters {
             sse_algorithm_ == "kms" ?
                 config.get<std::string>("vfs.s3.sse_kms_key_id").value() :
                 "")
+      , storage_class_(
+            config.get<std::string>("vfs.s3.storage_class", Config::must_find))
       , bucket_acl_str_(config.get<std::string>(
             "vfs.s3.bucket_canned_acl", Config::must_find))
       , object_acl_str_(config.get<std::string>(
@@ -326,6 +328,9 @@ struct S3Parameters {
   /** The server-side encryption key to use with the kms algorithm. */
   std::string sse_kms_key_id_;
 
+  /** The S3 storage class. */
+  std::string storage_class_;
+
   /** Names of values found in Aws::S3::Model::BucketCannedACL enumeration. */
   std::string bucket_acl_str_;
 
@@ -1378,6 +1383,9 @@ class S3 : FilesystemBase {
   /** The server-side encryption algorithm. */
   Aws::S3::Model::ServerSideEncryption sse_;
 
+  /** The storage class for a s3 upload request. */
+  Aws::S3::Model::StorageClass storage_class_;
+
   /** Protects file_buffers map */
   std::mutex file_buffers_mtx_;
 
