@@ -5,7 +5,7 @@
  *
  * The MIT License
  *
- * @copyright Copyright (c) 2017-2023 TileDB, Inc.
+ * @copyright Copyright (c) 2017-2024 TileDB, Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -64,8 +64,18 @@
 using namespace tiledb::common;
 using namespace tiledb::sm::stats;
 
-namespace tiledb {
-namespace sm {
+namespace tiledb::sm {
+
+/**
+ * Gets the effective memory budget for a query. This will be memory_budget if
+ * set, otherwise the value "sm.mem.total_budget" from config.
+ */
+static uint64_t get_effective_memory_budget(
+    const Config& config, optional<uint64_t> memory_budget) {
+  return memory_budget ?
+             *memory_budget :
+             config.get<uint64_t>("sm.mem.total_budget", Config::must_find);
+}
 
 /* ****************************** */
 /*   CONSTRUCTORS & DESTRUCTORS   */
@@ -74,21 +84,27 @@ namespace sm {
 Query::Query(
     StorageManager* storage_manager,
     shared_ptr<Array> array,
-    optional<std::string> fragment_name)
-    : query_memory_tracker_(
-          storage_manager->resources().create_memory_tracker())
+    optional<std::string> fragment_name,
+    optional<uint64_t> memory_budget)
+    : resources_(storage_manager->resources())
+    , stats_(resources_.stats().create_child("Query"))
+    , logger_(resources_.logger()->clone("Query", ++logger_id_))
+    , query_memory_tracker_(resources_.memory_tracker_manager().create_tracker(
+          get_effective_memory_budget(resources_.config(), memory_budget),
+          [stats = stats_]() {
+            stats->add_counter("memory_budget_exceeded", 1);
+          }))
     , array_shared_(array)
     , array_(array_shared_.get())
     , opened_array_(array->opened_array())
     , array_schema_(array->array_schema_latest_ptr())
+    , config_(resources_.config())
     , type_(array_->get_query_type())
     , layout_(
           (type_ == QueryType::READ || array_schema_->dense()) ?
               Layout::ROW_MAJOR :
               Layout::UNORDERED)
     , storage_manager_(storage_manager)
-    , stats_(storage_manager_->stats()->create_child("Query"))
-    , logger_(storage_manager->logger()->clone("Query", ++logger_id_))
     , dim_label_queries_(nullptr)
     , has_coords_buffer_(false)
     , has_zipped_coords_buffer_(false)
@@ -105,6 +121,7 @@ Query::Query(
     , is_dimension_label_ordered_read_(false)
     , dimension_label_increasing_(true)
     , fragment_size_(std::numeric_limits<uint64_t>::max())
+    , memory_budget_(memory_budget)
     , query_remote_buffer_storage_(std::nullopt)
     , default_channel_{make_shared<QueryChannel>(HERE(), *this, 0)} {
   assert(array->is_open());
@@ -128,9 +145,6 @@ Query::Query(
   callback_data_ = nullptr;
   status_ = QueryStatus::UNINITIALIZED;
 
-  if (storage_manager != nullptr)
-    config_ = storage_manager->config();
-
   // Set initial subarray configuration
   subarray_.set_config(type_, config_);
 
@@ -152,47 +166,52 @@ Query::~Query() {
 /* ****************************** */
 
 void Query::field_require_array_fixed(
-    const std::string_view origin, const char* name) {
-  if (!array_schema_->is_field(name)) {
+    const std::string_view origin, std::string_view field_name) {
+  if (!array_schema_->is_field(field_name.data())) {
     throw QueryException(
-        std::string{origin} + ": '" + name + "' is not an array field");
+        std::string{origin} + ": '" + std::string{field_name} +
+        "' is not an array field");
   }
-  if (array_schema_->var_size(name)) {
+  if (array_schema_->var_size(field_name.data())) {
     throw QueryException(
-        std::string{origin} + ": '" + name + "' is not fixed-sized");
+        std::string{origin} + ": '" + std::string{field_name} +
+        "' is not fixed-sized");
   }
 }
 
 void Query::field_require_array_variable(
-    const std::string_view origin, const char* name) {
-  if (!array_schema_->is_field(name)) {
+    const std::string_view origin, std::string_view field_name) {
+  if (!array_schema_->is_field(field_name.data())) {
     throw QueryException(
-        std::string{origin} + ": '" + name + "' is not an array field");
+        std::string{origin} + ": '" + std::string{field_name} +
+        "' is not an array field");
   }
-  if (!array_schema_->var_size(name)) {
+  if (!array_schema_->var_size(field_name.data())) {
     throw QueryException(
-        std::string{origin} + ": '" + name + "' is not variable-sized");
+        std::string{origin} + ": '" + std::string{field_name} +
+        "' is not variable-sized");
   }
 }
 
 void Query::field_require_array_nullable(
-    const std::string_view origin, const char* name) {
-  if (!array_schema_->attribute(name)) {
+    const std::string_view origin, std::string_view field_name) {
+  if (!array_schema_->attribute(field_name.data())) {
     throw QueryException(
-        std::string{origin} + ": '" + name +
+        std::string{origin} + ": '" + std::string{field_name} +
         "' is not the name of an attribute");
   }
-  if (!array_schema_->is_nullable(name)) {
+  if (!array_schema_->is_nullable(field_name.data())) {
     throw QueryException(
-        std::string{origin} + ": attribute '" + name + "' is not nullable");
+        std::string{origin} + ": attribute '" + std::string{field_name} +
+        "' is not nullable");
   }
 }
 
 void Query::field_require_array_nonnull(
-    const std::string_view origin, const char* name) {
-  if (array_schema_->is_nullable(name)) {
+    const std::string_view origin, std::string_view field_name) {
+  if (array_schema_->is_nullable(field_name.data())) {
     throw QueryException(
-        std::string(origin) + ": field '" + name +
+        std::string(origin) + ": field '" + std::string(field_name) +
         "' is not a nonnull array field");
   }
 }
@@ -200,16 +219,16 @@ void Query::field_require_array_nonnull(
 constexpr std::string_view origin_est_result_size{
     "query estimated result size"};
 
-FieldDataSize Query::internal_est_result_size(const char* name) {
+FieldDataSize Query::internal_est_result_size(std::string_view field_name) {
   if (type_ != QueryType::READ) {
     throw QueryException(
         std::string{origin_est_result_size} +
         ": operation currently supported only for read queries");
   }
   if (array_->is_remote() && !subarray_.est_result_size_computed()) {
-    auto rest_client = storage_manager_->rest_client();
+    auto rest_client = resources_.rest_client();
     if (rest_client == nullptr) {
-      throw QueryStatusException(
+      throw QueryException(
           "Error in query estimate result size; "
           "remote array with no rest client.");
     }
@@ -217,12 +236,13 @@ FieldDataSize Query::internal_est_result_size(const char* name) {
         rest_client->get_query_est_result_sizes(array_->array_uri(), this));
   }
   return subarray_.get_est_result_size(
-      name, &config_, storage_manager_->compute_tp());
+      field_name, &config_, &resources_.compute_tp());
 }
 
-FieldDataSize Query::get_est_result_size_fixed_nonnull(const char* name) {
-  field_require_array_fixed(origin_est_result_size, name);
-  if (name == constants::coords) {
+FieldDataSize Query::get_est_result_size_fixed_nonnull(
+    std::string_view field_name) {
+  field_require_array_fixed(origin_est_result_size, field_name);
+  if (field_name == constants::coords) {
     if (!array_schema_->domain().all_dims_same_type()) {
       throw QueryException(
           std::string{origin_est_result_size} +
@@ -236,38 +256,39 @@ FieldDataSize Query::get_est_result_size_fixed_nonnull(const char* name) {
           "in arrays with domains with variable-sized dimensions");
     }
   }
-  field_require_array_nonnull(origin_est_result_size, name);
-  return internal_est_result_size(name);
+  field_require_array_nonnull(origin_est_result_size, field_name);
+  return internal_est_result_size(field_name);
 }
 
-FieldDataSize Query::get_est_result_size_variable_nonnull(const char* name) {
-  field_require_array_variable(origin_est_result_size, name);
-  field_require_array_nonnull(origin_est_result_size, name);
-  return internal_est_result_size(name);
+FieldDataSize Query::get_est_result_size_variable_nonnull(
+    std::string_view field_name) {
+  field_require_array_variable(origin_est_result_size, field_name);
+  field_require_array_nonnull(origin_est_result_size, field_name);
+  return internal_est_result_size(field_name);
 }
 
-FieldDataSize Query::get_est_result_size_fixed_nullable(const char* name) {
-  field_require_array_fixed(origin_est_result_size, name);
-  field_require_array_nullable(origin_est_result_size, name);
-  return internal_est_result_size(name);
+FieldDataSize Query::get_est_result_size_fixed_nullable(
+    std::string_view field_name) {
+  field_require_array_fixed(origin_est_result_size, field_name);
+  field_require_array_nullable(origin_est_result_size, field_name);
+  return internal_est_result_size(field_name);
 }
 
-FieldDataSize Query::get_est_result_size_variable_nullable(const char* name) {
-  field_require_array_variable(origin_est_result_size, name);
-  field_require_array_nullable(origin_est_result_size, name);
-  return internal_est_result_size(name);
+FieldDataSize Query::get_est_result_size_variable_nullable(
+    std::string_view field_name) {
+  field_require_array_variable(origin_est_result_size, field_name);
+  field_require_array_nullable(origin_est_result_size, field_name);
+  return internal_est_result_size(field_name);
 }
 
 std::unordered_map<std::string, Subarray::ResultSize>
 Query::get_est_result_size_map() {
-  return subarray_.get_est_result_size_map(
-      &config_, storage_manager_->compute_tp());
+  return subarray_.get_est_result_size_map(&config_, &resources_.compute_tp());
 }
 
 std::unordered_map<std::string, Subarray::MemorySize>
 Query::get_max_mem_size_map() {
-  return subarray_.get_max_mem_size_map(
-      &config_, storage_manager_->compute_tp());
+  return subarray_.get_max_mem_size_map(&config_, &resources_.compute_tp());
 }
 
 Status Query::get_written_fragment_num(uint32_t* num) const {
@@ -426,20 +447,21 @@ Status Query::finalize() {
   }
 
   if (array_->is_remote() && type_ == QueryType::WRITE) {
-    auto rest_client = storage_manager_->rest_client();
-    if (rest_client == nullptr)
-      return logger_->status(Status_QueryError(
-          "Error in query finalize; remote array with no rest client."));
+    auto rest_client = resources_.rest_client();
+    if (rest_client == nullptr) {
+      throw QueryException(
+          "Failed to finalize query; remote array with no rest client.");
+    }
 
     if (layout_ == Layout::GLOBAL_ORDER) {
-      return logger_->status(Status_QueryError(
-          "Error in query finalize; remote global order writes are only "
-          "allowed to call submit_and_finalize to submit the last tile"));
+      throw QueryException(
+          "Failed to finalize query; remote global order writes are only "
+          "allowed to call submit_and_finalize to submit the last tile");
     }
     return rest_client->finalize_query_to_rest(array_->array_uri(), this);
   }
 
-  RETURN_NOT_OK(strategy_->finalize());
+  throw_if_not_ok(strategy_->finalize());
 
   status_ = QueryStatus::COMPLETED;
   return Status::Ok();
@@ -447,32 +469,33 @@ Status Query::finalize() {
 
 Status Query::submit_and_finalize() {
   if (type_ != QueryType::WRITE || layout_ != Layout::GLOBAL_ORDER) {
-    return logger_->status(
-        Status_QueryError("Error in query submit_and_finalize; Call valid only "
-                          "in global_order writes."));
+    throw QueryException(
+        "Failed to submit and finalize query; Call valid only in global_order "
+        "writes.");
   }
 
   // Check attribute/dimensions buffers completeness before query submits
-  RETURN_NOT_OK(check_buffers_correctness());
+  throw_if_not_ok(check_buffers_correctness());
 
   if (array_->is_remote()) {
-    auto rest_client = storage_manager_->rest_client();
-    if (rest_client == nullptr)
-      return logger_->status(
-          Status_QueryError("Error in query submit_and_finalize; remote array "
-                            "with no rest client."));
+    auto rest_client = resources_.rest_client();
+    if (rest_client == nullptr) {
+      throw QueryException(
+          "Failed to submit and finalize query; remote array with no rest "
+          "client.");
+    }
 
     if (status_ == QueryStatus::UNINITIALIZED) {
-      RETURN_NOT_OK(create_strategy());
+      throw_if_not_ok(create_strategy());
     }
     return rest_client->submit_and_finalize_query_to_rest(
         array_->array_uri(), this);
   }
 
   init();
-  RETURN_NOT_OK(storage_manager_->query_submit(this));
+  throw_if_not_ok(storage_manager_->query_submit(this));
 
-  RETURN_NOT_OK(strategy_->finalize());
+  throw_if_not_ok(strategy_->finalize());
   status_ = QueryStatus::COMPLETED;
 
   return Status::Ok();
@@ -641,14 +664,14 @@ void Query::init() {
       status_ == QueryStatus::INITIALIZED) {
     // Check if the array got closed
     if (array_ == nullptr || !array_->is_open()) {
-      throw QueryStatusException(
+      throw QueryException(
           "Cannot init query; The associated array is not open");
     }
 
     // Check if the array got re-opened with a different query type
     QueryType array_query_type{array_->get_query_type()};
     if (array_query_type != type_) {
-      throw QueryStatusException(
+      throw QueryException(
           "Cannot init query; Associated array query type does not match "
           "query type: (" +
           query_type_str(array_query_type) + " != " + query_type_str(type_) +
@@ -660,14 +683,14 @@ void Query::init() {
     // Create dimension label queries and remove labels from subarray.
     if (uses_dimension_labels()) {
       if (condition_.has_value()) {
-        throw QueryStatusException(
+        throw QueryException(
             "Cannot init query; Using query conditions and dimension labels "
             "together is not supported.");
       }
 
       // Check the layout is valid.
       if (layout_ == Layout::GLOBAL_ORDER) {
-        throw QueryStatusException(
+        throw QueryException(
             "Cannot init query; The global order layout is not supported "
             "when querying dimension labels");
       }
@@ -679,7 +702,7 @@ void Query::init() {
       if (!only_dim_label_query() && type_ == QueryType::READ &&
           !array_schema_->dense() && array_schema_->dim_num() > 1 &&
           !label_buffers_.empty()) {
-        throw QueryStatusException(
+        throw QueryException(
             "Cannot initialize query; Reading dimension label data is not "
             "yet supported on sparse arrays with multiple dimensions.");
       }
@@ -802,10 +825,7 @@ Status Query::process() {
     }
 
     throw_if_not_ok(parallel_for(
-        storage_manager_->compute_tp(),
-        0,
-        enmr_names.size(),
-        [&](const uint64_t i) {
+        &resources_.compute_tp(), 0, enmr_names.size(), [&](const uint64_t i) {
           array_->get_enumeration(enmr_names[i]);
           return Status::Ok();
         }));
@@ -918,11 +938,14 @@ void Query::set_processed_conditions(
 
 void Query::set_config(const Config& config) {
   if (!remote_query_ && status_ != QueryStatus::UNINITIALIZED) {
-    throw QueryStatusException(
+    throw QueryException(
         "[set_config] Cannot set config after initialization.");
   }
   config_.inherit(config);
 
+  query_memory_tracker_->refresh_memory_budget(
+      get_effective_memory_budget(config_, memory_budget_));
+
   // Refresh memory budget configuration.
   if (strategy_ != nullptr) {
     strategy_->refresh_config();
@@ -986,13 +1009,13 @@ Status Query::set_data_buffer(
   if (array_schema_->is_dim_label(name)) {
     // Check the query type is valid.
     if (type_ != QueryType::READ && type_ != QueryType::WRITE) {
-      throw QueryStatusException("[set_data_buffer] Unsupported query type.");
+      throw QueryException("[set_data_buffer] Unsupported query type.");
     }
 
     const bool exists = label_buffers_.find(name) != label_buffers_.end();
     if (status_ != QueryStatus::UNINITIALIZED && !exists &&
         !serialization_allow_new_attr) {
-      throw QueryStatusException(
+      throw QueryException(
           "[set_data_buffer] Cannot set buffer for new dimension label '" +
           name + "' after initialization");
     }
@@ -1136,13 +1159,12 @@ Status Query::set_offsets_buffer(
   if (array_schema_->is_dim_label(name)) {
     // Check the query type is valid.
     if (type_ != QueryType::READ && type_ != QueryType::WRITE) {
-      throw QueryStatusException(
-          "[set_offsets_buffer] Unsupported query type.");
+      throw QueryException("[set_offsets_buffer] Unsupported query type.");
     }
 
     // Check the dimension label is in fact variable length.
     if (!array_schema_->dimension_label(name).is_var()) {
-      throw QueryStatusException(
+      throw QueryException(
           "[set_offsets_buffer] Input dimension label '" + name +
           "' is fixed-sized");
     }
@@ -1151,7 +1173,7 @@ Status Query::set_offsets_buffer(
     const bool exists = label_buffers_.find(name) != label_buffers_.end();
     if (status_ != QueryStatus::UNINITIALIZED && !exists &&
         !serialization_allow_new_attr) {
-      throw QueryStatusException(
+      throw QueryException(
           "[set_offsets_buffer] Cannot set buffer for new dimension label '" +
           name + "' after initialization");
     }
@@ -1435,7 +1457,7 @@ void Query::set_subarray(const void* subarray) {
     case QueryType::WRITE:
     case QueryType::MODIFY_EXCLUSIVE:
       if (!array_schema_->dense()) {
-        throw QueryStatusException(
+        throw QueryException(
             "[set_subarray] Setting a subarray is not supported on "
             "sparse writes.");
       }
@@ -1443,7 +1465,7 @@ void Query::set_subarray(const void* subarray) {
 
     default:
 
-      throw QueryStatusException(
+      throw QueryException(
           "[set_subarray] Setting a subarray is not supported for query type "
           "'" +
           query_type_str(type_) + "'.");
@@ -1451,7 +1473,7 @@ void Query::set_subarray(const void* subarray) {
 
   // Check this isn't an already initialized query using dimension labels.
   if (status_ != QueryStatus::UNINITIALIZED) {
-    throw QueryStatusException(
+    throw QueryException(
         "[set_subarray] Setting a subarray on an already initialized  "
         "query is not supported.");
   }
@@ -1478,14 +1500,14 @@ void Query::set_subarray(const tiledb::sm::Subarray& subarray) {
     case QueryType::WRITE:
     case QueryType::MODIFY_EXCLUSIVE:
       if (!array_schema_->dense()) {
-        throw QueryStatusException(
+        throw QueryException(
             "[set_subarray] Setting a subarray is not supported on "
             "sparse writes.");
       }
       break;
 
     default:
-      throw QueryStatusException(
+      throw QueryException(
           "[set_subarray] Setting a subarray is not supported for query type "
           "'" +
           query_type_str(type_) + "'.");
@@ -1493,7 +1515,7 @@ void Query::set_subarray(const tiledb::sm::Subarray& subarray) {
 
   // Check the query has not been initialized.
   if (status_ != tiledb::sm::QueryStatus::UNINITIALIZED) {
-    throw QueryStatusException(
+    throw QueryException(
         "[set_subarray] Setting a subarray on an already initialized "
         "query is not supported.");
   }
@@ -1532,22 +1554,23 @@ Status Query::submit() {
   // Make sure fragment size is only set for global order.
   if (fragment_size_ != std::numeric_limits<uint64_t>::max() &&
       (layout_ != Layout::GLOBAL_ORDER || type_ != QueryType::WRITE)) {
-    throw QueryStatusException(
+    throw QueryException(
         "[submit] Fragment size is only supported for global order writes.");
   }
 
   // Check attribute/dimensions buffers completeness before query submits
-  RETURN_NOT_OK(check_buffers_correctness());
+  throw_if_not_ok(check_buffers_correctness());
 
   if (array_->is_remote()) {
-    auto rest_client = storage_manager_->rest_client();
-    if (rest_client == nullptr)
-      return logger_->status(Status_QueryError(
-          "Error in query submission; remote array with no rest client."));
+    auto rest_client = resources_.rest_client();
+    if (rest_client == nullptr) {
+      throw QueryException(
+          "Failed to submit query; remote array with no rest client.");
+    }
 
     if (status_ == QueryStatus::UNINITIALIZED && !only_dim_label_query() &&
         !subarray_.has_label_ranges()) {
-      RETURN_NOT_OK(create_strategy());
+      throw_if_not_ok(create_strategy());
 
       // Allocate remote buffer storage for global order writes if necessary.
       // If we cache an entire write a query may be uninitialized for N submits.
@@ -1557,13 +1580,14 @@ Status Query::submit() {
       }
     }
 
-    RETURN_NOT_OK(rest_client->submit_query_to_rest(array_->array_uri(), this));
+    throw_if_not_ok(
+        rest_client->submit_query_to_rest(array_->array_uri(), this));
 
     reset_coords_markers();
     return Status::Ok();
   }
   init();
-  RETURN_NOT_OK(storage_manager_->query_submit(this));
+  throw_if_not_ok(storage_manager_->query_submit(this));
 
   reset_coords_markers();
   return Status::Ok();
@@ -1708,7 +1732,7 @@ bool Query::use_refactored_sparse_unordered_with_dups_reader(
 }
 
 bool Query::non_overlapping_ranges() {
-  return subarray_.non_overlapping_ranges(storage_manager_->compute_tp());
+  return subarray_.non_overlapping_ranges(&resources_.compute_tp());
 }
 
 bool Query::is_dense() const {
@@ -1747,6 +1771,7 @@ Status Query::create_strategy(bool skip_checks_serialization) {
       storage_manager_,
       opened_array_,
       config_,
+      memory_budget_,
       buffers_,
       aggregate_buffers_,
       subarray_,
@@ -1955,7 +1980,7 @@ Status Query::check_buffer_names() {
       for (unsigned d = 0; d < array_schema_->dim_num(); d++) {
         auto dim = array_schema_->dimension_ptr(d);
         if (buffers_.count(dim->name()) == 0) {
-          throw QueryStatusException(
+          throw QueryException(
               "[check_buffer_names] Dimension buffer " + dim->name() +
               " is not set");
         }
@@ -2085,8 +2110,7 @@ void Query::copy_aggregates_data_to_user_buffer() {
 }
 
 RestClient* Query::rest_client() const {
-  return storage_manager_->rest_client();
+  return resources_.rest_client().get();
 }
 
-}  // namespace sm
-}  // namespace tiledb
+}  // namespace tiledb::sm
