@@ -5,7 +5,7 @@
  *
  * The MIT License
  *
- * @copyright Copyright (c) 2017-2023 TileDB, Inc.
+ * @copyright Copyright (c) 2017-2024 TileDB, Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -39,7 +39,6 @@
 #include <utility>
 #include <vector>
 
-#include "tiledb/common/logger_public.h"
 #include "tiledb/common/status.h"
 #include "tiledb/sm/array_schema/array_schema.h"
 #include "tiledb/sm/array_schema/dimension.h"
@@ -60,13 +59,12 @@
 
 using namespace tiledb::common;
 
-namespace tiledb {
-namespace sm {
+namespace tiledb::sm {
 
 /** Class for query status exceptions. */
-class QueryStatusException : public StatusException {
+class QueryException : public StatusException {
  public:
-  explicit QueryStatusException(const std::string& msg)
+  explicit QueryException(const std::string& msg)
       : StatusException("Query", msg) {
   }
 };
@@ -155,11 +153,14 @@ class Query {
    * writes.
    * @param fragment_base_uri Optional base name for new fragment. Only used for
    *     writes and only if fragment_uri is empty.
+   * @param memory_budget Total memory budget. If set to nullopt, the value
+   *     will be obtained from the sm.mem.total_budget config option.
    */
   Query(
       StorageManager* storage_manager,
       shared_ptr<Array> array,
-      optional<std::string> fragment_name = nullopt);
+      optional<std::string> fragment_name = nullopt,
+      optional<uint64_t> memory_budget = nullopt);
 
   /** Destructor. */
   virtual ~Query();
@@ -171,36 +172,95 @@ class Query {
   /*                 API               */
   /* ********************************* */
 
+  /**
+   * Require that a name be that of a fixed-size field from the source array.
+   *
+   * Validate these conditions and throw if any are not met.
+   * - The name is that of a field in the array
+   * - The field is fixed-sized
+   *
+   * @param origin The name of the operation that this validation is a part of
+   * @param field_name The name of a field
+   */
+  void field_require_array_fixed(
+      const std::string_view origin, std::string_view field_name);
+
+  /**
+   * Require that a name be that of a variable-size field from the source array.
+   *
+   * Validate these conditions and throw if any are not met.
+   * - The name is that of a field in the array
+   * - The field is variable-sized
+   *
+   * @param origin The name of the operation that this validation is a part of
+   * @param field_name The name of a field
+   */
+  void field_require_array_variable(
+      const std::string_view origin, std::string_view field_name);
+
+  /**
+   * Require that a field be a nullable field from the source array.
+   *
+   * Validate these conditions and throw if any are not met.
+   * - The name is that of an attribute of the source array
+   * - The attribute is nullable
+   *
+   * @param origin The name of the operation that this validation is a part of
+   * @param field_name The name of a field
+   */
+  void field_require_array_nullable(
+      const std::string_view origin, std::string_view field_name);
+
+  /**
+   * Require that a field be a nonnull field from the source array.
+   *
+   * Validate these conditions and throw if any are not met.
+   * - The name is that of a field in the array
+   * - The field is not nullable
+   *
+   * @param origin The name of the operation that this validation is a part of.
+   * @param field_name The name of a field
+   */
+  void field_require_array_nonnull(
+      const std::string_view origin, std::string_view field_name);
+
   /**
    * Gets the estimated result size (in bytes) for the input fixed-sized
    * attribute/dimension.
    */
-  Status get_est_result_size(const char* name, uint64_t* size);
+  FieldDataSize get_est_result_size_fixed_nonnull(std::string_view field_name);
 
   /**
    * Gets the estimated result size (in bytes) for the input var-sized
    * attribute/dimension.
    */
-  Status get_est_result_size(
-      const char* name, uint64_t* size_off, uint64_t* size_val);
+  FieldDataSize get_est_result_size_variable_nonnull(
+      std::string_view field_name);
 
   /**
    * Gets the estimated result size (in bytes) for the input fixed-sized,
    * nullable attribute.
    */
-  Status get_est_result_size_nullable(
-      const char* name, uint64_t* size_val, uint64_t* size_validity);
+  FieldDataSize get_est_result_size_fixed_nullable(std::string_view field_name);
 
   /**
    * Gets the estimated result size (in bytes) for the input var-sized,
    * nullable attribute.
    */
-  Status get_est_result_size_nullable(
-      const char* name,
-      uint64_t* size_off,
-      uint64_t* size_val,
-      uint64_t* size_validity);
+  FieldDataSize get_est_result_size_variable_nullable(
+      std::string_view field_name);
 
+ private:
+  /**
+   * Common part of all `est_result_size_*` functions, called after argument
+   * validation.
+   *
+   * @param field_name The name of a field
+   * @return estimated result size
+   */
+  FieldDataSize internal_est_result_size(std::string_view field_name);
+
+ public:
   /** Retrieves the number of written fragments. */
   Status get_written_fragment_num(uint32_t* num) const;
 
@@ -809,7 +869,7 @@ class Query {
       }
       return;
     }
-    throw QueryStatusException(
+    throw QueryException(
         "We currently only support a default channel for queries");
   }
 
@@ -844,13 +904,13 @@ class Query {
 
   inline std::shared_ptr<QueryChannel> aggegate_channel() {
     if (!has_aggregates()) {
-      throw QueryStatusException("Aggregate channel does not exist");
+      throw QueryException("Aggregate channel does not exist");
     }
     return aggregate_channel_;
   }
 
   /**
-   * Returns the REST client configured in the storage manager associated to
+   * Returns the REST client configured in the context resources associated to
    * this query
    */
   RestClient* rest_client() const;
@@ -860,6 +920,15 @@ class Query {
   /*         PRIVATE ATTRIBUTES        */
   /* ********************************* */
 
+  /** Resource used for operations. */
+  ContextResources& resources_;
+
+  /** The class stats. */
+  stats::Stats* stats_;
+
+  /** The class logger. */
+  shared_ptr<Logger> logger_;
+
   /** The query memory tracker. */
   shared_ptr<MemoryTracker> query_memory_tracker_;
 
@@ -906,12 +975,6 @@ class Query {
   /** The query strategy. */
   tdb_unique_ptr<IQueryStrategy> strategy_;
 
-  /** The class stats. */
-  stats::Stats* stats_;
-
-  /** The class logger. */
-  shared_ptr<Logger> logger_;
-
   /** UID of the logger instance */
   inline static std::atomic<uint64_t> logger_id_ = 0;
 
@@ -1030,6 +1093,12 @@ class Query {
    */
   uint64_t fragment_size_;
 
+  /**
+   * Memory budget. If set to nullopt, the value will be obtained from the
+   * sm.mem.total_budget config option.
+   */
+  optional<uint64_t> memory_budget_;
+
   /** Already written buffers. */
   std::unordered_set<std::string> written_buffers_;
 
@@ -1110,7 +1179,6 @@ class Query {
   void copy_aggregates_data_to_user_buffer();
 };
 
-}  // namespace sm
-}  // namespace tiledb
+}  // namespace tiledb::sm
 
 #endif  // TILEDB_QUERY_H
