@@ -48,6 +48,7 @@ namespace tiledb {
 namespace sm {
 
 class FragmentMetadata;
+class MemoryTracker;
 struct ResultCellSlab;
 class ResultTile;
 
@@ -55,6 +56,49 @@ enum class QueryConditionCombinationOp : uint8_t;
 
 class QueryCondition {
  public:
+  /** Class to pass query parameters to query condition processing. */
+  class Params {
+   public:
+    /* ********************************* */
+    /*     CONSTRUCTORS & DESTRUCTORS    */
+    /* ********************************* */
+
+    Params() = delete;
+
+    /** Constructor setting all parameters. */
+    Params(
+        shared_ptr<MemoryTracker> memory_tracker,
+        const ArraySchema& array_schema)
+        : memory_tracker_(memory_tracker)
+        , schema_(array_schema) {
+    }
+
+    /* ********************************* */
+    /*                API                */
+    /* ********************************* */
+
+    /** Returns the memory tracker. */
+    shared_ptr<MemoryTracker> GetMemoryTracker() const {
+      return memory_tracker_;
+    }
+
+    /** Returns the array schema. */
+    const ArraySchema& GetSchema() const {
+      return schema_;
+    }
+
+   private:
+    /* ********************************* */
+    /*         PRIVATE ATTRIBUTES        */
+    /* ********************************* */
+
+    /** Memory tracker. */
+    shared_ptr<MemoryTracker> memory_tracker_;
+
+    /** Array schema. */
+    const ArraySchema& schema_;
+  };
+
   /* ********************************* */
   /*     CONSTRUCTORS & DESTRUCTORS    */
   /* ********************************* */
@@ -192,7 +236,7 @@ class QueryCondition {
   /**
    * Applies this query condition to `result_cell_slabs`.
    *
-   * @param array_schema The array schema associated with `result_cell_slabs`.
+   * @param params Query condition parameters.
    * @param fragment_metadata The fragment metadata.
    * @param result_cell_slabs The cell slabs to filter. Mutated to remove cell
    *   slabs that do not meet the criteria in this query condition.
@@ -200,7 +244,7 @@ class QueryCondition {
    * @return Status
    */
   Status apply(
-      const ArraySchema& array_schema,
+      const QueryCondition::Params& params,
       const std::vector<shared_ptr<FragmentMetadata>>& fragment_metadata,
       std::vector<ResultCellSlab>& result_cell_slabs,
       uint64_t stride) const;
@@ -208,7 +252,7 @@ class QueryCondition {
   /**
    * Applies this query condition to a set of cells.
    *
-   * @param array_schema The array schema.
+   * @param params Query condition parameters.
    * @param result_tile The result tile to get the cells from.
    * @param start The start cell.
    * @param length The number of cells to process.
@@ -219,7 +263,7 @@ class QueryCondition {
    * @return Status
    */
   Status apply_dense(
-      const ArraySchema& array_schema,
+      const QueryCondition::Params& params,
       ResultTile* result_tile,
       const uint64_t start,
       const uint64_t length,
@@ -231,14 +275,14 @@ class QueryCondition {
   /**
    * Applies this query condition to a set of cells.
    *
-   * @param array_schema The array schema.
+   * @param params Query condition parameters.
    * @param result_tile The result tile to get the cells from.
    * @param result_bitmap The bitmap to use for results.
    * @return Status
    */
   template <typename BitmapType>
   Status apply_sparse(
-      const ArraySchema& array_schema,
+      const QueryCondition::Params& params,
       ResultTile& result_tile,
       tdb::pmr::vector<BitmapType>& result_bitmap);
 
@@ -354,7 +398,7 @@ class QueryCondition {
       const ByteVecValue& fill_value,
       const std::vector<ResultCellSlab>& result_cell_slabs,
       CombinationOp combination_op,
-      std::vector<uint8_t>& result_cell_bitmap) const;
+      tdb::pmr::vector<uint8_t>& result_cell_bitmap) const;
 
   /**
    * Applies a value node on primitive-typed result cell slabs.
@@ -379,7 +423,7 @@ class QueryCondition {
       const ByteVecValue& fill_value,
       const std::vector<ResultCellSlab>& result_cell_slabs,
       CombinationOp combination_op,
-      std::vector<uint8_t>& result_cell_bitmap) const;
+      tdb::pmr::vector<uint8_t>& result_cell_bitmap) const;
 
   /**
    * Applies a value node to filter result cells from the input
@@ -401,14 +445,14 @@ class QueryCondition {
       uint64_t stride,
       const std::vector<ResultCellSlab>& result_cell_slabs,
       CombinationOp combination_op,
-      std::vector<uint8_t>& result_cell_bitmap) const;
+      tdb::pmr::vector<uint8_t>& result_cell_bitmap) const;
 
   /**
    * Applies the query condition represented with the AST to
    * `result_cell_slabs`.
    *
    * @param node The node to apply.
-   * @param array_schema The array schema associated with `result_cell_slabs`.
+   * @param params Query condition parameters.
    * @param fragment_metadata The fragment metadata.
    * @param stride The stride between cells.
    * @param combination_op The combination op.
@@ -420,12 +464,12 @@ class QueryCondition {
   template <typename CombinationOp = std::logical_and<uint8_t>>
   void apply_tree(
       const tdb_unique_ptr<ASTNode>& node,
-      const ArraySchema& array_schema,
+      const QueryCondition::Params& params,
       const std::vector<shared_ptr<FragmentMetadata>>& fragment_metadata,
       uint64_t stride,
       const std::vector<ResultCellSlab>& result_cell_slabs,
       CombinationOp combination_op,
-      std::vector<uint8_t>& result_cell_bitmap) const;
+      tdb::pmr::vector<uint8_t>& result_cell_bitmap) const;
 
   /**
    * Applies a value node on a dense result tile,
@@ -518,7 +562,7 @@ class QueryCondition {
    * Applies the query condition represented with the AST to a set of cells.
    *
    * @param node The node to apply.
-   * @param array_schema The array schema.
+   * @param params Query condition parameters.
    * @param result_tile The result tile to get the cells from.
    * @param start The start cell.
    * @param src_cell The cell offset in the source tile.
@@ -531,7 +575,7 @@ class QueryCondition {
   template <typename CombinationOp = std::logical_and<uint8_t>>
   void apply_tree_dense(
       const tdb_unique_ptr<ASTNode>& node,
-      const ArraySchema& array_schema,
+      const QueryCondition::Params& params,
       ResultTile* result_tile,
       const uint64_t start,
       const uint64_t src_cell,
@@ -628,7 +672,7 @@ class QueryCondition {
    * Applies the query condition represented with the AST to a set of cells.
    *
    * @param node The node to apply.
-   * @param array_schema The array schema.
+   * @param params Query condition parameters.
    * @param result_tile The result tile to get the cells from.
    * @param combination_op The combination op.
    * @param result_bitmap The bitmap to use for results.
@@ -639,7 +683,7 @@ class QueryCondition {
       typename CombinationOp = std::logical_and<BitmapType>>
   void apply_tree_sparse(
       const tdb_unique_ptr<ASTNode>& node,
-      const ArraySchema& array_schema,
+      const QueryCondition::Params& params,
       ResultTile& result_tile,
       CombinationOp combination_op,
       tdb::pmr::vector<BitmapType>& result_bitmap) const;
