@@ -36,7 +36,6 @@
 #include <atomic>
 
 #include "tiledb/common/common.h"
-#include "tiledb/common/logger_public.h"
 #include "tiledb/common/status.h"
 #include "tiledb/sm/array_schema/dimension.h"
 #include "tiledb/sm/array_schema/dynamic_array.h"
@@ -159,6 +158,9 @@ class DenseReader : public ReaderBase, public IQueryStrategy {
   /** Total memory budget. */
   uint64_t memory_budget_;
 
+  /** Total memory budget if overridden by the query. */
+  optional<uint64_t> memory_budget_from_query_;
+
   /** Target upper memory limit for tiles. */
   uint64_t tile_upper_memory_limit_;
 
