@@ -31,6 +31,7 @@
  */
 
 #include "tiledb/sm/query/readers/reader_base.h"
+#include "tiledb/common/indexed_list.h"
 #include "tiledb/common/logger.h"
 #include "tiledb/sm/array/array.h"
 #include "tiledb/sm/array_schema/array_schema.h"
@@ -431,7 +432,7 @@ void ReaderBase::load_tile_offsets(
   const auto encryption_key = array_->encryption_key();
 
   throw_if_not_ok(parallel_for(
-      storage_manager_->compute_tp(),
+      &resources_.compute_tp(),
       0,
       relevant_fragments.size(),
       [&](const uint64_t i) {
@@ -449,7 +450,8 @@ void ReaderBase::load_tile_offsets(
           filtered_names.emplace_back(name);
         }
 
-        fragment->load_tile_offsets(*encryption_key, filtered_names);
+        fragment->loaded_metadata()->load_tile_offsets(
+            *encryption_key, filtered_names);
         return Status::Ok();
       }));
 }
@@ -461,7 +463,7 @@ void ReaderBase::load_tile_var_sizes(
   const auto encryption_key = array_->encryption_key();
 
   throw_if_not_ok(parallel_for(
-      storage_manager_->compute_tp(),
+      &resources_.compute_tp(),
       0,
       relevant_fragments.size(),
       [&](const uint64_t i) {
@@ -481,7 +483,8 @@ void ReaderBase::load_tile_var_sizes(
             continue;
           }
 
-          fragment->load_tile_var_sizes(*encryption_key, name);
+          fragment->loaded_metadata()->load_tile_var_sizes(
+              *encryption_key, name);
         }
 
         return Status::Ok();
@@ -495,7 +498,7 @@ void ReaderBase::load_tile_metadata(
   const auto encryption_key = array_->encryption_key();
 
   throw_if_not_ok(parallel_for(
-      storage_manager_->compute_tp(),
+      &resources_.compute_tp(),
       0,
       relevant_fragments.size(),
       [&](const uint64_t i) {
@@ -517,10 +520,14 @@ void ReaderBase::load_tile_metadata(
           }
         }
 
-        fragment->load_tile_max_values(*encryption_key, to_load);
-        fragment->load_tile_min_values(*encryption_key, to_load);
-        fragment->load_tile_sum_values(*encryption_key, to_load);
-        fragment->load_tile_null_count_values(*encryption_key, to_load);
+        fragment->loaded_metadata()->load_tile_max_values(
+            *encryption_key, to_load);
+        fragment->loaded_metadata()->load_tile_min_values(
+            *encryption_key, to_load);
+        fragment->loaded_metadata()->load_tile_sum_values(
+            *encryption_key, to_load);
+        fragment->loaded_metadata()->load_tile_null_count_values(
+            *encryption_key, to_load);
 
         return Status::Ok();
       }));
@@ -532,14 +539,15 @@ void ReaderBase::load_processed_conditions() {
 
   // Load all fragments in parallel.
   throw_if_not_ok(parallel_for(
-      storage_manager_->compute_tp(),
+      &resources_.compute_tp(),
       0,
       fragment_metadata_.size(),
       [&](const uint64_t i) {
         auto& fragment = fragment_metadata_[i];
 
         if (fragment->has_delete_meta()) {
-          fragment->load_processed_conditions(*encryption_key);
+          fragment->loaded_metadata()->load_processed_conditions(
+              *encryption_key);
         }
 
         return Status::Ok();
@@ -695,7 +703,7 @@ std::list<FilteredData> ReaderBase::read_tiles(
   stats_->add_counter("num_tiles_read", num_tiles_read);
 
   // Wait for the read tasks to finish.
-  auto statuses{storage_manager_->io_tp()->wait_all_status(read_tasks)};
+  auto statuses{resources_.io_tp().wait_all_status(read_tasks)};
   for (const auto& st : statuses) {
     throw_if_not_ok(st);
   }
@@ -841,7 +849,7 @@ Status ReaderBase::unfilter_tiles(
 
   // Compute parallelization parameters.
   uint64_t num_range_threads = 1;
-  const auto num_threads = storage_manager_->compute_tp()->concurrency_level();
+  const auto num_threads = resources_.compute_tp().concurrency_level();
   if (chunking && num_tiles < num_threads) {
     // Ceil the division between thread_num and num_tiles.
     num_range_threads = 1 + ((num_threads - 1) / num_tiles);
@@ -858,7 +866,7 @@ Status ReaderBase::unfilter_tiles(
 
   // Pre-compute chunk offsets.
   auto status = parallel_for(
-      storage_manager_->compute_tp(), 0, num_tiles, [&, this](uint64_t i) {
+      &resources_.compute_tp(), 0, num_tiles, [&, this](uint64_t i) {
         auto&& [st, tile_size, tile_var_size, tile_validity_size] =
             load_tile_chunk_data(
                 name,
@@ -882,7 +890,7 @@ Status ReaderBase::unfilter_tiles(
 
   // Unfilter all tiles/chunks in parallel using the precomputed offsets.
   status = parallel_for_2d(
-      storage_manager_->compute_tp(),
+      &resources_.compute_tp(),
       0,
       num_tiles,
       0,
@@ -973,7 +981,7 @@ Status ReaderBase::unfilter_tile(
         t_var->type(), array_schema_.version());
   }
 
-  auto concurrency_level = storage_manager_->compute_tp()->concurrency_level();
+  auto concurrency_level = resources_.compute_tp().concurrency_level();
 
   if (!validity_only) {
     // Unfiltered fixed data
@@ -993,7 +1001,7 @@ Status ReaderBase::unfilter_tile(
           t_min,
           t_max,
           concurrency_level,
-          storage_manager_->config()));
+          resources_.config()));
     }
 
     // Prevent processing past the end of chunks in case there are more
@@ -1012,7 +1020,7 @@ Status ReaderBase::unfilter_tile(
           tvar_min,
           tvar_max,
           concurrency_level,
-          storage_manager_->config()));
+          resources_.config()));
     }
   }
 
@@ -1036,7 +1044,7 @@ Status ReaderBase::unfilter_tile(
         tval_min,
         tval_max,
         concurrency_level,
-        storage_manager_->config()));
+        resources_.config()));
   }
 
   return Status::Ok();
@@ -1057,7 +1065,8 @@ uint64_t ReaderBase::get_attribute_tile_size(
   tile_size += fragment_metadata_[f]->tile_size(name, t);
 
   if (array_schema_.var_size(name)) {
-    tile_size += fragment_metadata_[f]->tile_var_size(name, t);
+    tile_size +=
+        fragment_metadata_[f]->loaded_metadata()->tile_var_size(name, t);
   }
 
   if (array_schema_.is_nullable(name)) {
@@ -1129,10 +1138,7 @@ ReaderBase::cache_dimension_label_data() {
   std::vector<const void*> non_empty_domains(fragment_metadata_.size());
   std::vector<uint64_t> frag_first_array_tile_idx(fragment_metadata_.size());
   throw_if_not_ok(parallel_for(
-      storage_manager_->compute_tp(),
-      0,
-      fragment_metadata_.size(),
-      [&](unsigned f) {
+      &resources_.compute_tp(), 0, fragment_metadata_.size(), [&](unsigned f) {
         non_empty_domains[f] =
             fragment_metadata_[f]->non_empty_domain()[0].data();
         auto ned = static_cast<const IndexType*>(non_empty_domains[f]);
@@ -1179,10 +1185,7 @@ void ReaderBase::validate_attribute_order(
   AttributeOrderValidator validator(
       attribute_name, fragment_metadata_.size(), query_memory_tracker_);
   throw_if_not_ok(parallel_for(
-      storage_manager_->compute_tp(),
-      0,
-      fragment_metadata_.size(),
-      [&](uint64_t f) {
+      &resources_.compute_tp(), 0, fragment_metadata_.size(), [&](uint64_t f) {
         validator.find_fragments_to_check(
             array_min_idx, array_max_idx, f, non_empty_domains);
 
@@ -1190,10 +1193,7 @@ void ReaderBase::validate_attribute_order(
       }));
 
   throw_if_not_ok(parallel_for(
-      storage_manager_->compute_tp(),
-      0,
-      fragment_metadata_.size(),
-      [&](int64_t f) {
+      &resources_.compute_tp(), 0, fragment_metadata_.size(), [&](int64_t f) {
         validator.validate_without_loading_tiles<IndexType, AttributeType>(
             index_dim,
             increasing_data,
@@ -1214,7 +1214,7 @@ void ReaderBase::validate_attribute_order(
 
     // Validate bounds not validated using tile data.
     throw_if_not_ok(parallel_for(
-        storage_manager_->compute_tp(),
+        &resources_.compute_tp(),
         0,
         fragment_metadata_.size(),
         [&](unsigned f) {
@@ -1369,3 +1369,10 @@ template void ReaderBase::validate_attribute_order<uint64_t>(
     std::vector<uint64_t>&);
 
 }  // namespace tiledb::sm
+
+template <>
+IndexedList<tiledb::sm::ResultTile>::IndexedList(
+    shared_ptr<tiledb::sm::MemoryTracker> memory_tracker)
+    : memory_tracker_(memory_tracker)
+    , list_(memory_tracker->get_resource(sm::MemoryType::RESULT_TILE)) {
+}
