@@ -5,7 +5,7 @@
  *
  * The MIT License
  *
- * @copyright Copyright (c) 2017-2023 TileDB, Inc.
+ * @copyright Copyright (c) 2017-2024 TileDB, Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -112,6 +112,13 @@ GlobalOrderWriter::GlobalOrderWriter(
 GlobalOrderWriter::~GlobalOrderWriter() {
 }
 
+GlobalOrderWriter::GlobalWriteState::GlobalWriteState(
+    shared_ptr<MemoryTracker> memory_tracker)
+    : last_tiles_(memory_tracker->get_resource(MemoryType::WRITER_TILE_DATA))
+    , last_var_offsets_(memory_tracker->get_resource(MemoryType::WRITER_DATA))
+    , cells_written_(memory_tracker->get_resource(MemoryType::WRITER_DATA)) {
+}
+
 /* ****************************** */
 /*               API              */
 /* ****************************** */
@@ -174,7 +181,7 @@ Status GlobalOrderWriter::alloc_global_write_state() {
     return logger_->status(
         Status_WriterError("Cannot initialize global write state; State not "
                            "properly finalized"));
-  global_write_state_.reset(new GlobalWriteState);
+  global_write_state_.reset(tdb_new(GlobalWriteState, query_memory_tracker_));
 
   // Alloc FragmentMetadata object
   global_write_state_->frag_meta_ = this->create_fragment_metadata();
@@ -201,7 +208,7 @@ Status GlobalOrderWriter::init_global_write_state() {
     auto last_tiles_it = global_write_state_->last_tiles_.emplace(
         std::piecewise_construct,
         std::forward_as_tuple(name),
-        std::forward_as_tuple());
+        std::forward_as_tuple(query_memory_tracker_));
     last_tiles_it.first->second.emplace_back(
         array_schema_,
         cell_num_per_tile,
@@ -239,7 +246,7 @@ GlobalOrderWriter::multipart_upload_state(bool client) {
   for (const auto& name : buffer_names()) {
     auto uri = meta->uri(name);
 
-    auto&& [st2, state] = storage_manager_->vfs()->multipart_upload_state(uri);
+    auto&& [st2, state] = resources_.vfs().multipart_upload_state(uri);
     RETURN_NOT_OK_TUPLE(st2, {});
     // If there is no entry for this uri, probably multipart upload is disabled
     // or no write was issued so far
@@ -250,8 +257,7 @@ GlobalOrderWriter::multipart_upload_state(bool client) {
 
     if (array_schema_.var_size(name)) {
       auto var_uri = meta->var_uri(name);
-      auto&& [st, var_state] =
-          storage_manager_->vfs()->multipart_upload_state(var_uri);
+      auto&& [st, var_state] = resources_.vfs().multipart_upload_state(var_uri);
       RETURN_NOT_OK_TUPLE(st, {});
       result[var_uri.remove_trailing_slash().last_path_part()] =
           std::move(*var_state);
@@ -259,7 +265,7 @@ GlobalOrderWriter::multipart_upload_state(bool client) {
     if (array_schema_.is_nullable(name)) {
       auto validity_uri = meta->validity_uri(name);
       auto&& [st, val_state] =
-          storage_manager_->vfs()->multipart_upload_state(validity_uri);
+          resources_.vfs().multipart_upload_state(validity_uri);
       RETURN_NOT_OK_TUPLE(st, {});
       result[validity_uri.remove_trailing_slash().last_path_part()] =
           std::move(*val_state);
@@ -281,8 +287,7 @@ Status GlobalOrderWriter::set_multipart_upload_state(
   // uri in this case holds only the buffer name
   auto absolute_uri =
       global_write_state_->frag_meta_->fragment_uri().join_path(uri);
-  return storage_manager_->vfs()->set_multipart_upload_state(
-      absolute_uri, state);
+  return resources_.vfs().set_multipart_upload_state(absolute_uri, state);
 }
 
 /* ****************************** */
@@ -321,10 +326,7 @@ Status GlobalOrderWriter::check_coord_dups() const {
   }
 
   auto status = parallel_for(
-      storage_manager_->compute_tp(),
-      1,
-      coords_info_.coords_num_,
-      [&](uint64_t i) {
+      &resources_.compute_tp(), 1, coords_info_.coords_num_, [&](uint64_t i) {
         // Check for duplicate in adjacent cells
         bool found_dup = true;
         for (unsigned d = 0; d < dim_num; ++d) {
@@ -421,7 +423,7 @@ Status GlobalOrderWriter::check_global_order() const {
 
   // Check if all coordinates are in global order in parallel
   auto status = parallel_for(
-      storage_manager_->compute_tp(),
+      &resources_.compute_tp(),
       0,
       coords_info_.coords_num_ - 1,
       [&](uint64_t i) {
@@ -472,7 +474,7 @@ Status GlobalOrderWriter::check_global_order_hilbert() const {
 
   // Check if all coordinates are in hilbert order in parallel
   auto status = parallel_for(
-      storage_manager_->compute_tp(),
+      &resources_.compute_tp(),
       0,
       coords_info_.coords_num_ - 1,
       [&](uint64_t i) {
@@ -502,13 +504,13 @@ void GlobalOrderWriter::clean_up() {
     // Cleanup the fragment we are currently writing. There is a chance that the
     // URI is empty if creating the first fragment had failed.
     if (!uri.empty()) {
-      throw_if_not_ok(storage_manager_->vfs()->remove_dir(uri));
+      throw_if_not_ok(resources_.vfs().remove_dir(uri));
     }
     global_write_state_.reset(nullptr);
 
     // Cleanup all fragments pending commit.
     for (auto& uri : frag_uris_to_commit_) {
-      throw_if_not_ok(storage_manager_->vfs()->remove_dir(uri));
+      throw_if_not_ok(resources_.vfs().remove_dir(uri));
     }
     frag_uris_to_commit_.clear();
   }
@@ -570,10 +572,7 @@ Status GlobalOrderWriter::compute_coord_dups(
 
   std::mutex mtx;
   auto status = parallel_for(
-      storage_manager_->compute_tp(),
-      1,
-      coords_info_.coords_num_,
-      [&](uint64_t i) {
+      &resources_.compute_tp(), 1, coords_info_.coords_num_, [&](uint64_t i) {
         // Check for duplicate in adjacent cells
         bool found_dup = true;
         for (unsigned d = 0; d < dim_num; ++d) {
@@ -702,7 +701,7 @@ Status GlobalOrderWriter::finalize_global_write_state() {
   // Write either one commit file or a consolidated commit file if multiple
   // fragments were written.
   if (frag_uris_to_commit_.size() == 0) {
-    RETURN_NOT_OK(storage_manager_->vfs()->touch(commit_uri));
+    throw_if_not_ok(resources_.vfs().touch(commit_uri));
   } else {
     std::vector<URI> commit_uris;
     commit_uris.reserve(frag_uris_to_commit_.size() + 1);
@@ -713,10 +712,7 @@ Status GlobalOrderWriter::finalize_global_write_state() {
 
     auto write_version = array_->array_schema_latest().write_version();
     Consolidator::write_consolidated_commits_file(
-        write_version,
-        array_->array_directory(),
-        commit_uris,
-        storage_manager_);
+        write_version, array_->array_directory(), commit_uris, resources_);
   }
 
   // Delete global write state
@@ -749,7 +745,8 @@ Status GlobalOrderWriter::global_write() {
     RETURN_CANCEL_OR_ERROR(compute_coord_dups(&coord_dups));
   }
 
-  std::unordered_map<std::string, WriterTileTupleVector> tiles;
+  tdb::pmr::unordered_map<std::string, WriterTileTupleVector> tiles(
+      query_memory_tracker_->get_resource(MemoryType::WRITER_TILE_DATA));
   RETURN_CANCEL_OR_ERROR(prepare_full_tiles(coord_dups, &tiles));
 
   // Find number of tiles and gather stats
@@ -853,33 +850,32 @@ Status GlobalOrderWriter::global_write_handle_last_tile() {
 void GlobalOrderWriter::nuke_global_write_state() {
   auto meta = global_write_state_->frag_meta_;
   throw_if_not_ok(close_files(meta));
-  throw_if_not_ok(storage_manager_->vfs()->remove_dir(meta->fragment_uri()));
+  throw_if_not_ok(resources_.vfs().remove_dir(meta->fragment_uri()));
   global_write_state_.reset(nullptr);
 }
 
 Status GlobalOrderWriter::prepare_full_tiles(
     const std::set<uint64_t>& coord_dups,
-    std::unordered_map<std::string, WriterTileTupleVector>* tiles) const {
+    tdb::pmr::unordered_map<std::string, WriterTileTupleVector>* tiles) const {
   auto timer_se = stats_->start_timer("prepare_tiles");
 
   // Initialize attribute and coordinate tiles
   for (const auto& it : buffers_) {
-    (*tiles).emplace(
+    tiles->emplace(
         std::piecewise_construct,
         std::forward_as_tuple(it.first),
-        std::forward_as_tuple());
+        std::forward_as_tuple(query_memory_tracker_));
   }
 
   auto num = buffers_.size();
-  auto status =
-      parallel_for(storage_manager_->compute_tp(), 0, num, [&](uint64_t i) {
-        auto buff_it = buffers_.begin();
-        std::advance(buff_it, i);
-        const auto& name = buff_it->first;
-        RETURN_CANCEL_OR_ERROR(
-            prepare_full_tiles(name, coord_dups, &(*tiles)[name]));
-        return Status::Ok();
-      });
+  auto status = parallel_for(&resources_.compute_tp(), 0, num, [&](uint64_t i) {
+    auto buff_it = buffers_.begin();
+    std::advance(buff_it, i);
+    const auto& name = buff_it->first;
+    RETURN_CANCEL_OR_ERROR(
+        prepare_full_tiles(name, coord_dups, &tiles->at(name)));
+    return Status::Ok();
+  });
 
   RETURN_NOT_OK(status);
 
@@ -919,7 +915,7 @@ Status GlobalOrderWriter::prepare_full_tiles_fixed(
   }
 
   // First fill the last tile
-  auto& last_tile = global_write_state_->last_tiles_[name][0];
+  auto& last_tile = global_write_state_->last_tiles_.at(name)[0];
   uint64_t cell_idx = 0;
   uint64_t last_tile_cell_idx =
       global_write_state_->cells_written_[name] % cell_num_per_tile;
@@ -1100,7 +1096,7 @@ Status GlobalOrderWriter::prepare_full_tiles_var(
     return Status::Ok();
 
   // First fill the last tile
-  auto& last_tile = global_write_state_->last_tiles_[name][0];
+  auto& last_tile = global_write_state_->last_tiles_.at(name)[0];
   auto& last_var_offset = global_write_state_->last_var_offsets_[name];
   uint64_t cell_idx = 0;
   uint64_t last_tile_cell_idx =
@@ -1378,7 +1374,7 @@ Status GlobalOrderWriter::prepare_full_tiles_var(
 uint64_t GlobalOrderWriter::num_tiles_to_write(
     uint64_t start,
     uint64_t tile_num,
-    std::unordered_map<std::string, WriterTileTupleVector>& tiles) {
+    tdb::pmr::unordered_map<std::string, WriterTileTupleVector>& tiles) {
   // Cache variables to prevent map lookups.
   const auto buf_names = buffer_names();
   std::vector<bool> var_size;
@@ -1390,7 +1386,7 @@ uint64_t GlobalOrderWriter::num_tiles_to_write(
   for (auto& name : buf_names) {
     var_size.emplace_back(array_schema_.var_size(name));
     nullable.emplace_back(array_schema_.is_nullable(name));
-    writer_tile_vectors.emplace_back(&tiles[name]);
+    writer_tile_vectors.emplace_back(&tiles.at(name));
   }
 
   // Make sure we don't write more than the desired fragment size.
