@@ -60,6 +60,7 @@
 #include "tiledb/common/logger.h"
 #include "tiledb/common/memory_tracker.h"
 #include "tiledb/sm/array/array.h"
+#include "tiledb/sm/config/config_iter.h"
 #include "tiledb/sm/enums/query_type.h"
 #include "tiledb/sm/group/group.h"
 #include "tiledb/sm/misc/constants.h"
@@ -148,6 +149,14 @@ Status RestClient::init(
   RETURN_NOT_OK(config_->get<bool>(
       "rest.resubmit_incomplete", &resubmit_incomplete_, &found));
 
+  load_headers(*config_);
+
+  if (auto payer =
+          config_->get<std::string>("rest.payer_namespace", Config::must_find);
+      !payer.empty()) {
+    extra_headers_["X-Payer"] = std::move(payer);
+  }
+
   return Status::Ok();
 }
 
@@ -1660,6 +1669,17 @@ RestClient::post_consolidation_plan_from_rest(
       serialization_type_, returned_data);
 }
 
+void RestClient::load_headers(const Config& cfg) {
+  for (auto iter = ConfigIter(cfg, constants::rest_header_prefix); !iter.end();
+       iter.next()) {
+    auto key = iter.param();
+    if (key.size() == 0) {
+      continue;
+    }
+    extra_headers_[key] = iter.value();
+  }
+}
+
 #else
 
 RestClient::RestClient() {
@@ -1831,6 +1851,10 @@ RestClient::post_consolidation_plan_from_rest(
   throw RestClientDisabledException();
 }
 
+void RestClient::load_headers(const Config&) {
+  throw RestClientDisabledException();
+}
+
 #endif  // TILEDB_SERIALIZATION
 
 }  // namespace tiledb::sm
