@@ -36,7 +36,6 @@
 #include <atomic>
 
 #include "tiledb/common/common.h"
-#include "tiledb/common/logger_public.h"
 #include "tiledb/common/thread_pool.h"
 #include "tiledb/sm/buffer/buffer.h"
 #include "tiledb/sm/config/config.h"
@@ -66,6 +65,23 @@ class Range;
 
 namespace tiledb::sm {
 
+struct FieldDataSize {
+  /**
+   * The size of fixed-length field data in bytes.
+   */
+  size_t fixed_;
+
+  /**
+   * The size of variable-length field data in bytes.
+   */
+  size_t variable_;
+
+  /**
+   * The size of validity data in bytes.
+   */
+  size_t validity_;
+};
+
 class Array;
 class ArraySchema;
 class OpenedArray;
@@ -413,13 +429,9 @@ class Subarray {
    * @param label_name The name of the dimension label to add the range to.
    * @param start The range start.
    * @param end The range end.
-   * @param stride The range stride.
    */
   void add_label_range(
-      const std::string& label_name,
-      const void* start,
-      const void* end,
-      const void* stride);
+      const std::string& label_name, const void* start, const void* end);
 
   /**
    * Adds a variable-sized range along the dimension with the given index for
@@ -443,13 +455,11 @@ class Subarray {
       uint32_t dim_idx, Range&& range, const bool read_range_oob_error = true);
 
   /**
-   * Adds a range to the subarray on the input dimension by index,
-   * in the form of (start, end, stride).
+   * Adds a range to the subarray on the input dimension by index.
    * The range components must be of the same type as the domain type of the
    * underlying array.
    */
-  void add_range(
-      unsigned dim_idx, const void* start, const void* end, const void* stride);
+  void add_range(unsigned dim_idx, const void* start, const void* end);
 
   /**
    * @brief Set point ranges from an array
@@ -505,16 +515,12 @@ class Subarray {
   void add_range_unsafe(uint32_t dim_idx, const Range& range);
 
   /**
-   * Adds a range to the (read/write) query on the input dimension by name,
-   * in the form of (start, end, stride).
+   * Adds a range to the (read/write) query on the input dimension by name.
    * The range components must be of the same type as the domain type of the
    * underlying array.
    */
   void add_range_by_name(
-      const std::string& dim_name,
-      const void* start,
-      const void* end,
-      const void* stride);
+      const std::string& dim_name, const void* start, const void* end);
 
   /**
    * Adds a variable-sized range to the (read/write) query on the input
@@ -551,22 +557,19 @@ class Subarray {
   const std::string& get_label_name(const uint32_t dim_index) const;
 
   /**
-   * Retrieves a range from a dimension label name in the form (start, end,
-   * stride).
+   * Retrieves a range from a dimension label name.
    *
    * @param label_name The name of the dimension label to retrieve the range
    *     from.
    * @param range_idx The id of the range to retrieve.
    * @param start The range start to retrieve.
    * @param end The range end to retrieve.
-   * @param stride The range stride to retrieve.
    */
   void get_label_range(
       const std::string& label_name,
       uint64_t range_idx,
       const void** start,
-      const void** end,
-      const void** stride) const;
+      const void** end) const;
 
   /**
    * Retrieves the number of ranges of the subarray for the given dimension
@@ -617,20 +620,18 @@ class Subarray {
       const std::string& dim_name, uint64_t* range_num) const;
 
   /**
-   * Retrieves a range from a dimension name in the form (start, end, stride).
+   * Retrieves a range from a dimension name.
    *
    * @param dim_name The dimension to retrieve the range from.
    * @param range_idx The id of the range to retrieve.
    * @param start The range start to retrieve.
    * @param end The range end to retrieve.
-   * @param stride The range stride to retrieve.
    */
   void get_range_from_name(
       const std::string& dim_name,
       uint64_t range_idx,
       const void** start,
-      const void** end,
-      const void** stride) const;
+      const void** end) const;
 
   /**
    * Retrieves a range's sizes for a variable-length dimension name
@@ -879,22 +880,6 @@ class Subarray {
   /** Retrieves the number of ranges on the given dimension index. */
   void get_range_num(uint32_t dim_idx, uint64_t* range_num) const;
 
-  /**
-   * Retrieves a range from a dimension index in the form (start, end, stride).
-   *
-   * @param dim_idx The dimension to retrieve the range from.
-   * @param range_idx The id of the range to retrieve.
-   * @param start The range start to retrieve.
-   * @param end The range end to retrieve.
-   * @param stride The range stride to retrieve.
-   */
-  void get_range(
-      unsigned dim_idx,
-      uint64_t range_idx,
-      const void** start,
-      const void** end,
-      const void** stride) const;
-
   /**
    * ``True`` if the specified dimension is set by default.
    *
@@ -919,96 +904,30 @@ class Subarray {
    */
   bool is_unary() const;
 
-  /**
-   * Gets the estimated result size (in bytes) for the input fixed-sized
-   * attribute/dimension.
-   */
-  void get_est_result_size_internal(
-      const char* name,
-      uint64_t* size,
-      const Config* config,
-      ThreadPool* compute_tp);
-
-  /**
-   * Gets the estimated result size (in bytes) for the input var-sized
-   * attribute/dimension.
-   */
-  void get_est_result_size(
-      const char* name,
-      uint64_t* size_off,
-      uint64_t* size_val,
-      const Config* config,
-      ThreadPool* compute_tp);
-
-  /**
-   * Gets the estimated result size (in bytes) for the input fixed-sized,
-   * nullable attribute.
-   */
-  void get_est_result_size_nullable(
-      const char* name,
-      uint64_t* size,
-      uint64_t* size_validity,
-      const Config* config,
-      ThreadPool* compute_tp);
-
-  /**
-   * Gets the estimated result size (in bytes) for the input var-sized,
-   * nullable attribute.
-   */
-  void get_est_result_size_nullable(
-      const char* name,
-      uint64_t* size_off,
-      uint64_t* size_val,
-      uint64_t* size_validity,
-      const Config* config,
-      ThreadPool* compute_tp);
-
   /** Returns whether the estimated result size has been computed or not. */
   bool est_result_size_computed();
 
-  /*
-   * Gets the maximum memory required to produce the result (in bytes)
-   * for the input fixed-sized attribute/dimension.
+  /**
+   * The estimated result size in bytes for a field.
+   *
+   * This function handles all fields. The field may be a dimension or
+   * attribute, fixed-length or variable, nullable or not. If a particular
+   * size is not relevant to the field type, then it's returned as zero.
    */
-  void get_max_memory_size(
-      const char* name,
-      uint64_t* size,
+  FieldDataSize get_est_result_size(
+      std::string_view field_name,
       const Config* config,
       ThreadPool* compute_tp);
 
   /**
-   * Gets the maximum memory required to produce the result (in bytes)
-   * for the input var-sized attribute/dimension.
+   * The maximum memory in bytes required for a field.
+   *
+   * This function handles all fields. The field may be a dimension or
+   * attribute, fixed-length or variable, nullable or not. If a particular
+   * size is not relevant to the field type, then it's returned as zero.
    */
-  void get_max_memory_size(
-      const char* name,
-      uint64_t* size_off,
-      uint64_t* size_val,
-      const Config* config,
-      ThreadPool* compute_tp);
-
-  /*
-   * Gets the maximum memory required to produce the result (in bytes)
-   * for the input fixed-sized, nullable attribute.
-   */
-  void get_max_memory_size_nullable(
-      const char* name,
-      uint64_t* size,
-      uint64_t* size_validity,
-      const Config* config,
-      ThreadPool* compute_tp);
-
-  /**
-   * Gets the maximum memory required to produce the result (in bytes)
-   * for the input var-sized, nullable attribute.
-   */
-  void get_max_memory_size_nullable(
-      const char* name,
-      uint64_t* size_off,
-      uint64_t* size_val,
-      uint64_t* size_validity,
-      const Config* config,
-      ThreadPool* compute_tp);
+  FieldDataSize get_max_memory_size(
+      const char* name, const Config* config, ThreadPool* compute_tp);
 
   /**
    * Returns the range coordinates (for all dimensions) given a flattened
@@ -1018,12 +937,6 @@ class Subarray {
    */
   std::vector<uint64_t> get_range_coords(uint64_t range_idx) const;
 
-  /**
-   * Advances the input range coords to the next coords along the
-   * subarray range layout.
-   */
-  void get_next_range_coords(std::vector<uint64_t>* range_coords) const;
-
   /**
    * Returns a subarray consisting of the dimension ranges specified by
    * the input.
